# Copyright 2016 EasyStack.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from magnumclient.common import utils as magnum_utils
from magnumclient import exceptions
from magnumclient.i18n import _

from osc_lib.command import command
from osc_lib import utils


CLUSTER_ATTRIBUTES = [
    'status',
    'health_status',
    'cluster_template_id',
    'node_addresses',
    'uuid',
    'stack_id',
    'status_reason',
    'created_at',
    'updated_at',
    'coe_version',
    'labels',
    'labels_overridden',
    'labels_skipped',
    'labels_added',
    'fixed_network',
    'fixed_subnet',
    'floating_ip_enabled',
    'faults',
    'keypair',
    'api_address',
    'master_addresses',
    'master_lb_enabled',
    'create_timeout',
    'node_count',
    'discovery_url',
    'docker_volume_size',
    'master_count',
    'container_version',
    'name',
    'master_flavor_id',
    'flavor_id',
    'health_status_reason',
    'project_id',
]


class CreateCluster(command.Command):
    _description = _("Create a cluster")

    def get_parser(self, prog_name):
        parser = super(CreateCluster, self).get_parser(prog_name)
        # NOTE: All arguments are positional and, if not provided
        # with a default, required.
        parser.add_argument('--cluster-template',
                            dest='cluster_template',
                            required=True,
                            metavar='<cluster-template>',
                            help='ID or name of the cluster template.')
        parser.add_argument('--discovery-url',
                            dest='discovery_url',
                            metavar='<discovery-url>',
                            help=('Specifies custom delivery url for '
                                  'node discovery.'))
        parser.add_argument('--docker-volume-size',
                            dest='docker_volume_size',
                            type=int,
                            metavar='<docker-volume-size>',
                            help=('The size in GB for the docker volume to '
                                  'use.'))
        parser.add_argument('--labels',
                            metavar='<KEY1=VALUE1,KEY2=VALUE2;KEY3=VALUE3...>',
                            action='append',
                            help=_('Arbitrary labels in the form of key=value '
                                   'pairs to associate with a cluster '
                                   'template. May be used multiple times.'))
        parser.add_argument('--keypair',
                            default=None,
                            metavar='<keypair>',
                            help='UUID or name of the keypair to use.')
        parser.add_argument('--master-count',
                            dest='master_count',
                            type=int,
                            default=1,
                            metavar='<master-count>',
                            help='The number of master nodes for the cluster.')
        parser.add_argument('name',
                            metavar='<name>',
                            help='Name of the cluster to create.')
        parser.add_argument('--node-count',
                            dest='node_count',
                            type=int,
                            default=1,
                            metavar='<node-count>',
                            help='The cluster node count.')
        parser.add_argument('--timeout',
                            type=int,
                            default=60,
                            metavar='<timeout>',
                            help=('The timeout for cluster creation time. The '
                                  'default is 60 minutes.'))
        parser.add_argument(
            '--master-flavor',
            dest='master_flavor',
            metavar='<master-flavor>',
            help=_('The nova flavor name or UUID to use when launching the '
                   'master node of the Cluster.'))
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            help=_('The nova flavor name or UUID to use when launching the '
                   'Cluster.'))
        parser.add_argument(
            '--fixed-network',
            dest='fixed_network',
            metavar='<fixed-network>',
            help=_('The private Neutron network name to connect to this '
                   'Cluster template.'))
        parser.add_argument(
            '--fixed-subnet',
            dest='fixed_subnet',
            metavar='<fixed-subnet>',
            help=_('The private Neutron subnet name to connect to Cluster.'))
        parser.add_argument(
            '--floating-ip-enabled',
            dest='floating_ip_enabled',
            default=[],
            action='append_const',
            const=True,
            help=_('Indicates whether created Clusters should have a '
                   'floating ip.'))
        parser.add_argument(
            '--floating-ip-disabled',
            dest='floating_ip_enabled',
            action='append_const',
            const=False,
            help=_('Disables floating ip creation on the new Cluster'))
        parser.add_argument(
            '--merge-labels',
            dest='merge_labels',
            action='store_true',
            default=False,
            help=_('The labels provided will be merged with the labels '
                   'configured in the specified cluster template.'))
        parser.add_argument(
            '--master-lb-enabled',
            dest='master_lb_enabled',
            action='append_const',
            default=[],
            const=True,
            help=_('Enable master LB creation on the new cluster'))
        parser.add_argument(
            '--master-lb-disabled',
            dest='master_lb_enabled',
            action='append_const',
            const=False,
            help=_('Disable master LB creation on the new cluster'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        args = {
            'cluster_template_id': parsed_args.cluster_template,
            'create_timeout': parsed_args.timeout,
            'discovery_url': parsed_args.discovery_url,
            'keypair': parsed_args.keypair,
            'master_count': parsed_args.master_count,
            'name': parsed_args.name,
            'node_count': parsed_args.node_count,
        }

        if len(parsed_args.floating_ip_enabled) > 1:
            raise exceptions.InvalidAttribute(
                '--floating-ip-enabled and '
                '--floating-ip-disabled are '
                'mutually exclusive and '
                'should be specified only once.')
        elif len(parsed_args.floating_ip_enabled) == 1:
            args['floating_ip_enabled'] = parsed_args.floating_ip_enabled[0]

        if parsed_args.labels is not None:
            args['labels'] = magnum_utils.handle_labels(parsed_args.labels)

        if parsed_args.docker_volume_size is not None:
            args['docker_volume_size'] = parsed_args.docker_volume_size

        if parsed_args.master_flavor is not None:
            args['master_flavor_id'] = parsed_args.master_flavor

        if parsed_args.flavor is not None:
            args['flavor_id'] = parsed_args.flavor

        if parsed_args.fixed_network is not None:
            args["fixed_network"] = parsed_args.fixed_network

        if parsed_args.fixed_subnet is not None:
            args["fixed_subnet"] = parsed_args.fixed_subnet

        if parsed_args.merge_labels:
            # We are only sending this if it's True. This
            # way we avoid breaking older APIs.
            args["merge_labels"] = parsed_args.merge_labels

        if parsed_args.master_lb_enabled:
            args["master_lb_enabled"] = parsed_args.master_lb_enabled

        if len(parsed_args.master_lb_enabled) > 1:
            raise exceptions.InvalidAttribute(
                '--master-lb-enabled and '
                '--master-lb-disabled are '
                'mutually exclusive and '
                'should be specified only once.')
        elif len(parsed_args.master_lb_enabled) == 1:
            args['master_lb_enabled'] = parsed_args.master_lb_enabled[0]
            if (not args['master_lb_enabled'] and
                    parsed_args.master_count > 1):
                raise exceptions.InvalidAttribute(
                    'Master node count can only be one if master '
                    'loadbalancer is disabled.')

        cluster = mag_client.clusters.create(**args)
        print("Request to create cluster %s accepted"
              % cluster.uuid)


class DeleteCluster(command.Command):
    _description = _("Delete a cluster")

    def get_parser(self, prog_name):
        parser = super(DeleteCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            nargs='+',
            metavar='<cluster>',
            help='ID or name of the cluster(s) to delete.')

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        for cluster in parsed_args.cluster:
            mag_client.clusters.delete(cluster)
            print("Request to delete cluster %s has been accepted." % cluster)


class ListCluster(command.Lister):
    _description = _("List clusters")

    def get_parser(self, prog_name):
        parser = super(ListCluster, self).get_parser(prog_name)

        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Maximum number of clusters to return'))
        parser.add_argument(
            '--sort-key',
            metavar='<sort-key>',
            help=_('Column to sort results by'))
        parser.add_argument(
            '--sort-dir',
            metavar='<sort-dir>',
            choices=['desc', 'asc'],
            help=_('Direction to sort. "asc" or "desc".'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        columns = [
            'uuid', 'name', 'keypair', 'node_count', 'master_count', 'status',
            'health_status']
        clusters = mag_client.clusters.list(limit=parsed_args.limit,
                                            sort_key=parsed_args.sort_key,
                                            sort_dir=parsed_args.sort_dir)
        return (
            columns,
            (utils.get_item_properties(c, columns) for c in clusters)
        )


class ShowCluster(command.ShowOne):
    _description = _("Show a Cluster")

    def get_parser(self, prog_name):
        parser = super(ShowCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster to show.')
            )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        columns = CLUSTER_ATTRIBUTES

        mag_client = self.app.client_manager.container_infra
        cluster = mag_client.clusters.get(parsed_args.cluster)

        return (columns, utils.get_item_properties(cluster, columns))


class UpdateCluster(command.Command):
    _description = _("Update a Cluster")

    def get_parser(self, prog_name):
        parser = super(UpdateCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('The name or UUID of cluster to update'))

        parser.add_argument(
            'op',
            metavar='<op>',
            choices=['add', 'replace', 'remove'],
            help=_("Operations: one of 'add', 'replace' or 'remove'"))

        parser.add_argument(
            'attributes',
            metavar='<path=value>',
            nargs='+',
            action='append',
            default=[],
            help=_(
                "Attributes to add/replace or remove (only PATH is necessary "
                "on remove)"))

        parser.add_argument(
            '--rollback',
            action='store_true',
            dest='rollback',
            default=False,
            help=_('Rollback cluster on update failure.'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra

        patch = magnum_utils.args_array_to_patch(parsed_args.op,
                                                 parsed_args.attributes[0])

        mag_client.clusters.update(parsed_args.cluster,
                                   patch)
        print("Request to update cluster %s has been accepted." %
              parsed_args.cluster)


class ConfigCluster(command.Command):
    _description = _("Get Configuration for a Cluster")

    def get_parser(self, prog_name):
        parser = super(ConfigCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('The name or UUID of cluster to update'))
        parser.add_argument(
            '--dir',
            metavar='<dir>',
            default='.',
            help=_('Directory to save the certificate and config files.'))
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help=_('Overwrite files if existing.'))
        parser.add_argument(
            '--output-certs',
            action='store_true',
            dest='output_certs',
            default=False,
            help=_('Output certificates in separate files.'))
        parser.add_argument(
            '--use-certificate',
            action='store_true',
            dest='use_certificate',
            default=True,
            help=_('Use certificate in config files.'))
        parser.add_argument(
            '--use-keystone',
            action='store_true',
            dest='use_keystone',
            default=False,
            help=_('Use Keystone token in config files.'))

        return parser

    def take_action(self, parsed_args):
        """Configure native client to access cluster.

        You can source the output of this command to get the native client of
        the corresponding COE configured to access the cluster.

        """
        if parsed_args.use_keystone:
            parsed_args.use_certificate = False
        if not parsed_args.use_certificate:
            parsed_args.use_keystone = True

        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra

        parsed_args.dir = os.path.abspath(parsed_args.dir)
        cluster = mag_client.clusters.get(parsed_args.cluster)
        if cluster.api_address is None:
            self.log.warning("WARNING: The cluster's api_address is"
                             " not known yet.")

        cluster_template = mag_client.cluster_templates.get(
            cluster.cluster_template_id)

        tls = self._fetch_tls(
            mag_client=mag_client,
            cluster_uuid=cluster.uuid,
            tls_required=(not cluster_template.tls_disabled),
            certkey_required=(not parsed_args.use_keystone)
        )

        if parsed_args.output_certs:
            self._write_certs(
                certs=tls,
                path=parsed_args.dir,
                force=parsed_args.force
            )

        print(magnum_utils.config_cluster(
            cluster, cluster_template, parsed_args.dir,
            force=parsed_args.force, certs=tls,
            use_keystone=parsed_args.use_keystone))

    def _fetch_tls(
            self,
            mag_client,
            cluster_uuid,
            tls_required,
            certkey_required=True,
    ):
        if not tls_required:
            return {}
        opts = {
            'cluster_uuid': cluster_uuid,
        }
        tls = {
            'ca': mag_client.certificates.get(**opts).pem
        }

        if certkey_required:
            csr = magnum_utils.generate_csr_and_key()
            opts['csr'] = csr['csr']
            tls['cert'] = mag_client.certificates.create(**opts).pem
            tls['key'] = csr['key']
        return tls

    def _write_certs(self, certs, path, force):
        for k in ('key', 'cert', 'ca'):
            fname = "%s/%s.pem" % (path, k)
            if os.path.exists(fname):
                if not force:
                    raise Exception("File %s exists, aborting." % fname)
                os.remove(fname)

            if k not in certs:
                # key and cert aren't always generated
                continue
            with open(fname, "w") as f:
                f.write(certs[k])


class ResizeCluster(command.Command):
    _description = _("Resize a Cluster")

    def get_parser(self, prog_name):
        parser = super(ResizeCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('The name or UUID of cluster to update'))

        parser.add_argument(
            'node_count',
            type=int,
            help=_("Desired node count of the cluser."))

        parser.add_argument(
            '--nodes-to-remove',
            metavar='<Server UUID>',
            action='append',
            help=_('Server ID of the nodes to be removed. Repeat to add '
                   'more server ID'))

        parser.add_argument(
            '--nodegroup',
            metavar='<nodegroup>',
            help=_('The name or UUID of the nodegroup of current cluster.'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        cluster = mag_client.clusters.get(parsed_args.cluster)

        mag_client.clusters.resize(cluster.uuid,
                                   parsed_args.node_count,
                                   parsed_args.nodes_to_remove,
                                   parsed_args.nodegroup)
        print("Request to resize cluster %s has been accepted." %
              parsed_args.cluster)


class UpgradeCluster(command.Command):
    _description = _("Upgrade a Cluster")

    def get_parser(self, prog_name):
        parser = super(UpgradeCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('The name or UUID of cluster to update'))

        parser.add_argument(
            'cluster_template',
            help=_("The new cluster template ID will be upgraded to."))

        parser.add_argument(
            '--max-batch-size',
            metavar='<max_batch_size>',
            type=int,
            default=1,
            help=_("The max batch size for upgrading each time."))

        parser.add_argument(
            '--nodegroup',
            metavar='<nodegroup>',
            help=_('The name or UUID of the nodegroup of current cluster.'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        cluster = mag_client.clusters.get(parsed_args.cluster)

        mag_client.clusters.upgrade(cluster.uuid,
                                    parsed_args.cluster_template,
                                    parsed_args.max_batch_size,
                                    parsed_args.nodegroup)
        print("Request to upgrade cluster %s has been accepted." %
              parsed_args.cluster)



# Copyright 2015 IBM Corp
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import testtools
from testtools import matchers

from magnumclient import exceptions
from magnumclient.tests import utils
from magnumclient.v1 import cluster_templates


CLUSTERTEMPLATE1 = {
    'id': 123,
    'uuid': '66666666-7777-8888-9999-000000000001',
    'name': 'clustertemplate1',
    'image_id': 'clustertemplate1-image',
    'master_flavor_id': 'm1.tiny',
    'flavor_id': 'm1.small',
    'external_network_id': 'd1f02cfb-d27f-4068-9332-84d907cb0e21',
    'fixed_network': 'private',
    'fixed_subnet': 'private-subnet',
    'network_driver': 'libnetwork',
    'volume_driver': 'rexray',
    'dns_nameserver': '8.8.1.1',
    'docker_volume_size': '71',
    'docker_storage_driver': 'devicemapper',
    'coe': 'swarm',
    'http_proxy': 'http_proxy',
    'https_proxy': 'https_proxy',
    'no_proxy': 'no_proxy',
    'labels': 'key1=val1,key11=val11',
    'tls_disabled': False,
    'public': False,
    'registry_enabled': False,
    'master_lb_enabled': True,
    'floating_ip_enabled': True,
    'hidden': False
    }

CLUSTERTEMPLATE2 = {
    'id': 124,
    'uuid': '66666666-7777-8888-9999-000000000002',
    'name': 'clustertemplate2',
    'image_id': 'clustertemplate2-image',
    'flavor_id': 'm2.small',
    'master_flavor_id': 'm2.tiny',
    'external_network_id': 'd1f02cfb-d27f-4068-9332-84d907cb0e22',
    'fixed_network': 'private2',
    'network_driver': 'flannel',
    'volume_driver': 'cinder',
    'dns_nameserver': '8.8.1.2',
    'docker_volume_size': '71',
    'docker_storage_driver': 'overlay',
    'coe': 'kubernetes',
    'labels': 'key2=val2,key22=val22',
    'tls_disabled': True,
    'public': True,
    'registry_enabled': True}

CREATE_CLUSTERTEMPLATE = copy.deepcopy(CLUSTERTEMPLATE1)
del CREATE_CLUSTERTEMPLATE['id']
del CREATE_CLUSTERTEMPLATE['uuid']

UPDATED_CLUSTERTEMPLATE = copy.deepcopy(CLUSTERTEMPLATE1)
NEW_NAME = 'newcluster'
UPDATED_CLUSTERTEMPLATE['name'] = NEW_NAME

fake_responses = {
    '/v1/clustertemplates':
    {
        'GET': (
            {},
            {'clustertemplates': [CLUSTERTEMPLATE1, CLUSTERTEMPLATE2]},
        ),
        'POST': (
            {},
            CREATE_CLUSTERTEMPLATE,
        ),
    },
    '/v1/clustertemplates/%s' % CLUSTERTEMPLATE1['id']:
    {
        'GET': (
            {},
            CLUSTERTEMPLATE1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_CLUSTERTEMPLATE,
        ),
    },
    '/v1/clustertemplates/%s' % CLUSTERTEMPLATE1['name']:
    {
        'GET': (
            {},
            CLUSTERTEMPLATE1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_CLUSTERTEMPLATE,
        ),
    },
    '/v1/clustertemplates/detail':
    {
        'GET': (
            {},
            {'clustertemplates': [CLUSTERTEMPLATE1, CLUSTERTEMPLATE2]},
        ),
    },
    '/v1/clustertemplates/?limit=2':
    {
        'GET': (
            {},
            {'clustertemplates': [CLUSTERTEMPLATE1, CLUSTERTEMPLATE2]},
        ),
    },
    '/v1/clustertemplates/?marker=%s' % CLUSTERTEMPLATE2['uuid']:
    {
        'GET': (
            {},
            {'clustertemplates': [CLUSTERTEMPLATE1, CLUSTERTEMPLATE2]},
        ),
    },
    '/v1/clustertemplates/?limit=2&marker=%s' % CLUSTERTEMPLATE2['uuid']:
    {
        'GET': (
            {},
            {'clustertemplates': [CLUSTERTEMPLATE1, CLUSTERTEMPLATE2]},
        ),
    },
    '/v1/clustertemplates/?sort_dir=asc':
    {
        'GET': (
            {},
            {'clustertemplates': [CLUSTERTEMPLATE1, CLUSTERTEMPLATE2]},
        ),
    },
    '/v1/clustertemplates/?sort_key=uuid':
    {
        'GET': (
            {},
            {'clustertemplates': [CLUSTERTEMPLATE1, CLUSTERTEMPLATE2]},
        ),
    },
    '/v1/clustertemplates/?sort_key=uuid&sort_dir=desc':
    {
        'GET': (
            {},
            {'clustertemplates': [CLUSTERTEMPLATE2, CLUSTERTEMPLATE1]},
        ),
    },
}


class ClusterTemplateManagerTest(testtools.TestCase):

    def setUp(self):
        super(ClusterTemplateManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = cluster_templates.ClusterTemplateManager(self.api)

    def test_clustertemplate_list(self):
        clustertemplates = self.mgr.list()
        expect = [
            ('GET', '/v1/clustertemplates', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(clustertemplates, matchers.HasLength(2))

    def _test_clustertemplate_list_with_filters(
            self, limit=None, marker=None,
            sort_key=None, sort_dir=None,
            detail=False, expect=[]):
        clustertemplates_filter = self.mgr.list(limit=limit,
                                                marker=marker,
                                                sort_key=sort_key,
                                                sort_dir=sort_dir,
                                                detail=detail)
        self.assertEqual(expect, self.api.calls)
        self.assertThat(clustertemplates_filter, matchers.HasLength(2))

    def test_clustertemplate_list_with_detail(self):
        expect = [
            ('GET', '/v1/clustertemplates/detail', {}, None),
        ]
        self._test_clustertemplate_list_with_filters(
            detail=True,
            expect=expect)

    def test_clustertemplate_list_with_limit(self):
        expect = [
            ('GET', '/v1/clustertemplates/?limit=2', {}, None),
        ]
        self._test_clustertemplate_list_with_filters(
            limit=2,
            expect=expect)

    def test_clustertemplate_list_with_marker(self):
        expect = [
            ('GET',
             '/v1/clustertemplates/?marker=%s' % CLUSTERTEMPLATE2['uuid'],
             {},
             None),
        ]
        self._test_clustertemplate_list_with_filters(
            marker=CLUSTERTEMPLATE2['uuid'],
            expect=expect)

    def test_clustertemplate_list_with_marker_limit(self):
        expect = [
            ('GET',
             '/v1/clustertemplates/?limit=2&marker=%s' %
             CLUSTERTEMPLATE2['uuid'],
             {},
             None),
        ]
        self._test_clustertemplate_list_with_filters(
            limit=2, marker=CLUSTERTEMPLATE2['uuid'],
            expect=expect)

    def test_clustertemplate_list_with_sort_dir(self):
        expect = [
            ('GET', '/v1/clustertemplates/?sort_dir=asc', {}, None),
        ]
        self._test_clustertemplate_list_with_filters(
            sort_dir='asc',
            expect=expect)

    def test_clustertemplate_list_with_sort_key(self):
        expect = [
            ('GET', '/v1/clustertemplates/?sort_key=uuid', {}, None),
        ]
        self._test_clustertemplate_list_with_filters(
            sort_key='uuid',
            expect=expect)

    def test_clustertemplate_list_with_sort_key_dir(self):
        expect = [
            ('GET',
             '/v1/clustertemplates/?sort_key=uuid&sort_dir=desc',
             {},
             None),
        ]
        self._test_clustertemplate_list_with_filters(
            sort_key='uuid', sort_dir='desc',
            expect=expect)

    def test_clustertemplate_show_by_id(self):
        cluster_template = self.mgr.get(CLUSTERTEMPLATE1['id'])
        expect = [
            ('GET',
             '/v1/clustertemplates/%s' % CLUSTERTEMPLATE1['id'],
             {},
             None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(CLUSTERTEMPLATE1['name'],
                         cluster_template.name)
        self.assertEqual(CLUSTERTEMPLATE1['image_id'],
                         cluster_template.image_id)
        self.assertEqual(CLUSTERTEMPLATE1['docker_volume_size'],
                         cluster_template.docker_volume_size)
        self.assertEqual(CLUSTERTEMPLATE1['docker_storage_driver'],
                         cluster_template.docker_storage_driver)
        self.assertEqual(CLUSTERTEMPLATE1['fixed_network'],
                         cluster_template.fixed_network)
        self.assertEqual(CLUSTERTEMPLATE1['fixed_subnet'],
                         cluster_template.fixed_subnet)
        self.assertEqual(CLUSTERTEMPLATE1['coe'],
                         cluster_template.coe)
        self.assertEqual(CLUSTERTEMPLATE1['http_proxy'],
                         cluster_template.http_proxy)
        self.assertEqual(CLUSTERTEMPLATE1['https_proxy'],
                         cluster_template.https_proxy)
        self.assertEqual(CLUSTERTEMPLATE1['no_proxy'],
                         cluster_template.no_proxy)
        self.assertEqual(CLUSTERTEMPLATE1['network_driver'],
                         cluster_template.network_driver)
        self.assertEqual(CLUSTERTEMPLATE1['volume_driver'],
                         cluster_template.volume_driver)
        self.assertEqual(CLUSTERTEMPLATE1['labels'],
                         cluster_template.labels)
        self.assertEqual(CLUSTERTEMPLATE1['tls_disabled'],
                         cluster_template.tls_disabled)
        self.assertEqual(CLUSTERTEMPLATE1['public'],
                         cluster_template.public)
        self.assertEqual(CLUSTERTEMPLATE1['registry_enabled'],
                         cluster_template.registry_enabled)
        self.assertEqual(CLUSTERTEMPLATE1['master_lb_enabled'],
                         cluster_template.master_lb_enabled)
        self.assertEqual(CLUSTERTEMPLATE1['floating_ip_enabled'],
                         cluster_template.floating_ip_enabled)
        self.assertEqual(CLUSTERTEMPLATE1['hidden'],
                         cluster_template.hidden)

    def test_clustertemplate_show_by_name(self):
        cluster_template = self.mgr.get(CLUSTERTEMPLATE1['name'])
        expect = [
            ('GET',
             '/v1/clustertemplates/%s' % CLUSTERTEMPLATE1['name'],
             {},
             None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(CLUSTERTEMPLATE1['name'],
                         cluster_template.name)
        self.assertEqual(CLUSTERTEMPLATE1['image_id'],
                         cluster_template.image_id)
        self.assertEqual(CLUSTERTEMPLATE1['docker_volume_size'],
                         cluster_template.docker_volume_size)
        self.assertEqual(CLUSTERTEMPLATE1['docker_storage_driver'],
                         cluster_template.docker_storage_driver)
        self.assertEqual(CLUSTERTEMPLATE1['fixed_network'],
                         cluster_template.fixed_network)
        self.assertEqual(CLUSTERTEMPLATE1['fixed_subnet'],
                         cluster_template.fixed_subnet)
        self.assertEqual(CLUSTERTEMPLATE1['coe'],
                         cluster_template.coe)
        self.assertEqual(CLUSTERTEMPLATE1['http_proxy'],
                         cluster_template.http_proxy)
        self.assertEqual(CLUSTERTEMPLATE1['https_proxy'],
                         cluster_template.https_proxy)
        self.assertEqual(CLUSTERTEMPLATE1['no_proxy'],
                         cluster_template.no_proxy)
        self.assertEqual(CLUSTERTEMPLATE1['network_driver'],
                         cluster_template.network_driver)
        self.assertEqual(CLUSTERTEMPLATE1['volume_driver'],
                         cluster_template.volume_driver)
        self.assertEqual(CLUSTERTEMPLATE1['labels'],
                         cluster_template.labels)
        self.assertEqual(CLUSTERTEMPLATE1['tls_disabled'],
                         cluster_template.tls_disabled)
        self.assertEqual(CLUSTERTEMPLATE1['public'],
                         cluster_template.public)
        self.assertEqual(CLUSTERTEMPLATE1['registry_enabled'],
                         cluster_template.registry_enabled)
        self.assertEqual(CLUSTERTEMPLATE1['master_lb_enabled'],
                         cluster_template.master_lb_enabled)
        self.assertEqual(CLUSTERTEMPLATE1['floating_ip_enabled'],
                         cluster_template.floating_ip_enabled)
        self.assertEqual(CLUSTERTEMPLATE1['hidden'],
                         cluster_template.hidden)

    def test_clustertemplate_create(self):
        cluster_template = self.mgr.create(**CREATE_CLUSTERTEMPLATE)
        expect = [
            ('POST', '/v1/clustertemplates', {}, CREATE_CLUSTERTEMPLATE),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(cluster_template)
        self.assertEqual(CLUSTERTEMPLATE1['docker_volume_size'],
                         cluster_template.docker_volume_size)
        self.assertEqual(CLUSTERTEMPLATE1['docker_storage_driver'],
                         cluster_template.docker_storage_driver)

    def test_clustertemplate_create_with_keypair(self):
        cluster_template_with_keypair = dict()
        cluster_template_with_keypair.update(CREATE_CLUSTERTEMPLATE)
        cluster_template_with_keypair['keypair_id'] = 'test_key'

        cluster_template = self.mgr.create(**cluster_template_with_keypair)
        expect = [
            ('POST', '/v1/clustertemplates', {},
             cluster_template_with_keypair),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(cluster_template)
        self.assertEqual(CLUSTERTEMPLATE1['docker_volume_size'],
                         cluster_template.docker_volume_size)
        self.assertEqual(CLUSTERTEMPLATE1['docker_storage_driver'],
                         cluster_template.docker_storage_driver)

    def test_clustertemplate_create_with_docker_volume_size(self):
        cluster_template_with_docker_volume_size = dict()
        cluster_template_with_docker_volume_size.update(CREATE_CLUSTERTEMPLATE)
        cluster_template_with_docker_volume_size['docker_volume_size'] = 11

        cluster_template = self.mgr.create(
            **cluster_template_with_docker_volume_size)
        expect = [
            ('POST', '/v1/clustertemplates', {},
             cluster_template_with_docker_volume_size),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(cluster_template)
        self.assertEqual(CLUSTERTEMPLATE1['docker_volume_size'],
                         cluster_template.docker_volume_size)
        self.assertEqual(CLUSTERTEMPLATE1['docker_storage_driver'],
                         cluster_template.docker_storage_driver)

    def test_clustertemplate_create_fail(self):
        CREATE_CLUSTERTEMPLATE_FAIL = copy.deepcopy(CREATE_CLUSTERTEMPLATE)
        CREATE_CLUSTERTEMPLATE_FAIL["wrong_key"] = "wrong"
        self.assertRaisesRegex(
            exceptions.InvalidAttribute,
            ("Key must be in %s" %
             ','.join(cluster_templates.CREATION_ATTRIBUTES)),
            self.mgr.create, **CREATE_CLUSTERTEMPLATE_FAIL)
        self.assertEqual([], self.api.calls)

    def test_clustertemplate_delete_by_id(self):
        cluster_template = self.mgr.delete(CLUSTERTEMPLATE1['id'])
        expect = [
            ('DELETE',
             '/v1/clustertemplates/%s' % CLUSTERTEMPLATE1['id'],
             {},
             None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(cluster_template)

    def test_clustertemplate_delete_by_name(self):
        cluster_template = self.mgr.delete(CLUSTERTEMPLATE1['name'])
        expect = [
            ('DELETE',
             '/v1/clustertemplates/%s' % CLUSTERTEMPLATE1['name'],
             {},
             None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(cluster_template)

    def test_clustertemplate_update(self):
        patch = {'op': 'replace',
                 'value': NEW_NAME,
                 'path': '/name'}
        cluster_template = self.mgr.update(id=CLUSTERTEMPLATE1['id'],
                                           patch=patch)
        expect = [
            ('PATCH',
             '/v1/clustertemplates/%s' % CLUSTERTEMPLATE1['id'],
             {},
             patch),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(NEW_NAME, cluster_template.name)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils
from magnumclient.i18n import _


def _show_quota(quota):
    utils.print_dict(quota._info)


@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help=_('The last quota UUID of the previous page; '
                  'displays list of quotas after "marker".'))
@utils.arg('--limit',
           metavar='<limit>',
           type=int,
           help=_('Maximum number of quotas to return.'))
@utils.arg('--sort-key',
           metavar='<sort-key>',
           help=_('Column to sort results by.'))
@utils.arg('--sort-dir',
           metavar='<sort-dir>',
           choices=['desc', 'asc'],
           help=_('Direction to sort. "asc" or "desc".'))
@utils.arg('--all-tenants',
           action='store_true',
           default=False,
           help=_('Flag to indicate list all tenant quotas.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_quotas_list(cs, args):
    """Print a list of available quotas."""
    quotas = cs.quotas.list(marker=args.marker,
                            limit=args.limit,
                            sort_key=args.sort_key,
                            sort_dir=args.sort_dir,
                            all_tenants=args.all_tenants)
    columns = ['project_id', 'resource', 'hard_limit']
    utils.print_list(quotas, columns,
                     {'versions': magnum_utils.print_list_field('versions')},
                     sortby_index=None)


@utils.arg('--project-id',
           required=True,
           metavar='<project-id>',
           help=_('Project Id.'))
@utils.arg('--resource',
           required=True,
           metavar='<resource>',
           help=_('Resource name.'))
@utils.arg('--hard-limit',
           metavar='<hard-limit>',
           type=int,
           default=1,
           help=_('Max resource limit.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_quotas_create(cs, args):
    """Create a quota."""

    opts = dict()
    opts['project_id'] = args.project_id
    opts['resource'] = args.resource
    opts['hard_limit'] = args.hard_limit

    try:
        quota = cs.quotas.create(**opts)
        _show_quota(quota)
    except Exception as e:
        print("Create quota for project_id %(id)s resource %(res)s failed: "
              "%(e)s" % {'id': args.project_id,
                         'res': args.resource,
                         'e': e.details})


@utils.arg('--project-id',
           required=True,
           metavar='<project-id>',
           help=_('Project ID.'))
@utils.arg('--resource',
           required=True,
           metavar='<resource>',
           help=_('Resource name'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_quotas_delete(cs, args):
    """Delete specified resource quota."""
    try:
        cs.quotas.delete(args.project_id, args.resource)
        print("Request to delete quota for project id %(id)s and resource "
              "%(res)s has been accepted." % {
                  'id': args.project_id, 'res': args.resource})
    except Exception as e:
        print("Quota delete failed for project id %(id)s and resource "
              "%(res)s :%(e)s" % {'id': args.project_id,
                                  'res': args.resource,
                                  'e': e.details})


@utils.arg('--project-id',
           required=True,
           metavar='<project-id>',
           help=_('Project ID.'))
@utils.arg('--resource',
           required=True,
           metavar='<resource>',
           help=_('Resource name'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_quotas_show(cs, args):
    """Show details about the given project resource quota."""
    quota = cs.quotas.get(args.project_id, args.resource)
    _show_quota(quota)


@utils.arg('--project-id',
           required=True,
           metavar='<project-id>',
           help=_('Project Id.'))
@utils.arg('--resource',
           required=True,
           metavar='<resource>',
           help=_('Resource name.'))
@utils.arg('--hard-limit',
           metavar='<hard-limit>',
           type=int,
           default=1,
           help=_('Max resource limit.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_quotas_update(cs, args):
    """Update information about the given project resource quota."""
    patch = dict()
    patch['project_id'] = args.project_id
    patch['resource'] = args.resource
    patch['hard_limit'] = args.hard_limit

    quota = cs.quotas.update(args.project_id, args.resource, patch)
    _show_quota(quota)



# -*- coding: utf-8 -*-
#
# Copyright 2012 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography import x509
from cryptography.x509.oid import NameOID
from oslo_serialization import base64
from oslo_serialization import jsonutils

from magnumclient import exceptions as exc
from magnumclient.i18n import _


def common_filters(marker=None, limit=None, sort_key=None, sort_dir=None):
    """Generate common filters for any list request.

    :param marker: entity ID from which to start returning entities.
    :param limit: maximum number of entities to return.
    :param sort_key: field to use for sorting.
    :param sort_dir: direction of sorting: 'asc' or 'desc'.
    :returns: list of string filters.
    """
    filters = []
    if isinstance(limit, int):
        filters.append('limit=%s' % limit)
    if marker is not None:
        filters.append('marker=%s' % marker)
    if sort_key is not None:
        filters.append('sort_key=%s' % sort_key)
    if sort_dir is not None:
        filters.append('sort_dir=%s' % sort_dir)
    return filters


def split_and_deserialize(string):
    """Split and try to JSON deserialize a string.

    Gets a string with the KEY=VALUE format, split it (using '=' as the
    separator) and try to JSON deserialize the VALUE.
    :returns: A tuple of (key, value).
    """
    try:
        key, value = string.split("=", 1)
    except ValueError:
        raise exc.CommandError(_('Attributes must be a list of '
                                 'PATH=VALUE not "%s"') % string)
    try:
        value = jsonutils.loads(value)
    except ValueError:
        pass

    return (key, value)


def args_array_to_patch(op, attributes):
    patch = []
    for attr in attributes:
        # Sanitize
        if not attr.startswith('/'):
            attr = '/' + attr
        if op in ['add', 'replace']:
            path, value = split_and_deserialize(attr)
            if path == "/labels" or path == "/health_status_reason":
                a = []
                a.append(value)
                value = str(handle_labels(a))
                patch.append({'op': op, 'path': path, 'value': value})
            else:
                patch.append({'op': op, 'path': path, 'value': value})

        elif op == "remove":
            # For remove only the key is needed
            patch.append({'op': op, 'path': attr})
        else:
            raise exc.CommandError(_('Unknown PATCH operation: %s') % op)
    return patch


def handle_labels(labels):
    labels = format_labels(labels)
    if 'mesos_slave_executor_env_file' in labels:
        environment_variables_data = handle_json_from_file(
            labels['mesos_slave_executor_env_file'])
        labels['mesos_slave_executor_env_variables'] = jsonutils.dumps(
            environment_variables_data)
    return labels


def format_labels(lbls, parse_comma=True):
    '''Reformat labels into dict of format expected by the API.'''

    if not lbls:
        return {}

    if parse_comma:
        # expect multiple invocations of --labels but fall back
        # to either , or ; delimited if only one --labels is specified
        if len(lbls) == 1 and lbls[0].count('=') > 1:
            lbls = lbls[0].replace(';', ',').split(',')

    labels = {}
    for lbl in lbls:
        try:
            (k, v) = lbl.split(('='), 1)
        except ValueError:
            raise exc.CommandError(_('labels must be a list of KEY=VALUE '
                                     'not %s') % lbl)
        if k not in labels:
            labels[k] = v
        else:
            labels[k] += ",%s" % v

    return labels


def print_list_field(field):
    return lambda obj: ', '.join(getattr(obj, field))


def handle_json_from_file(json_arg):
    """Attempts to read JSON file by the file url.

    :param json_arg: May be a file name containing the JSON.
    :returns: A list or dictionary parsed from JSON.
    """

    try:
        with open(json_arg, 'r') as f:
            json_arg = f.read().strip()
            json_arg = jsonutils.loads(json_arg)
    except IOError as e:
        err = _("Cannot get JSON from file '%(file)s'. "
                "Error: %(err)s") % {'err': e, 'file': json_arg}
        raise exc.InvalidAttribute(err)
    except ValueError as e:
        err = (_("For JSON: '%(string)s', error: '%(err)s'") %
               {'err': e, 'string': json_arg})
        raise exc.InvalidAttribute(err)

    return json_arg


def config_cluster(cluster, cluster_template, cfg_dir, force=False,
                   certs=None, use_keystone=False, direct_output=False):
    """Return and write configuration for the given cluster."""
    if cluster_template.coe == 'kubernetes':
        return _config_cluster_kubernetes(cluster, cluster_template, cfg_dir,
                                          force, certs, use_keystone,
                                          direct_output)
    elif (cluster_template.coe == 'swarm'
          or cluster_template.coe == 'swarm-mode'):
        return _config_cluster_swarm(cluster, cluster_template, cfg_dir,
                                     force, certs)


def _config_cluster_kubernetes(cluster, cluster_template, cfg_dir,
                               force=False, certs=None, use_keystone=False,
                               direct_output=False):
    """Return and write configuration for the given kubernetes cluster."""
    cfg_file = "%s/config" % cfg_dir
    if cluster_template.tls_disabled or certs is None:
        cfg = ("apiVersion: v1\n"
               "clusters:\n"
               "- cluster:\n"
               "    server: %(api_address)s\n"
               "  name: %(name)s\n"
               "contexts:\n"
               "- context:\n"
               "    cluster: %(name)s\n"
               "    user: %(name)s\n"
               "  name: %(name)s\n"
               "current-context: %(name)s\n"
               "kind: Config\n"
               "preferences: {}\n"
               "users:\n"
               "- name: %(name)s\n"
               % {'name': cluster.name, 'api_address': cluster.api_address})
    else:
        if not use_keystone:
            cfg = ("apiVersion: v1\n"
                   "clusters:\n"
                   "- cluster:\n"
                   "    certificate-authority-data: %(ca)s\n"
                   "    server: %(api_address)s\n"
                   "  name: %(name)s\n"
                   "contexts:\n"
                   "- context:\n"
                   "    cluster: %(name)s\n"
                   "    user: admin\n"
                   "  name: default\n"
                   "current-context: default\n"
                   "kind: Config\n"
                   "preferences: {}\n"
                   "users:\n"
                   "- name: admin\n"
                   "  user:\n"
                   "    client-certificate-data: %(cert)s\n"
                   "    client-key-data: %(key)s\n"
                   % {'name': cluster.name,
                      'api_address': cluster.api_address,
                      'key': base64.encode_as_text(certs['key']),
                      'cert': base64.encode_as_text(certs['cert']),
                      'ca': base64.encode_as_text(certs['ca'])})
        else:
            cfg = ("apiVersion: v1\n"
                   "clusters:\n"
                   "- cluster:\n"
                   "    certificate-authority-data: %(ca)s\n"
                   "    server: %(api_address)s\n"
                   "  name: %(name)s\n"
                   "contexts:\n"
                   "- context:\n"
                   "    cluster: %(name)s\n"
                   "    user: openstackuser\n"
                   "  name: openstackuser@kubernetes\n"
                   "current-context: openstackuser@kubernetes\n"
                   "kind: Config\n"
                   "preferences: {}\n"
                   "users:\n"
                   "- name: openstackuser\n"
                   "  user:\n"
                   "    exec:\n"
                   "      command: /bin/bash\n"
                   "      apiVersion: client.authentication.k8s.io/v1beta1\n"
                   "      args:\n"
                   "      - -c\n"
                   "      - >\n"
                   "        if [ -z ${OS_TOKEN} ]; then\n"
                   "            echo 'Error: Missing OpenStack credential from environment variable $OS_TOKEN' > /dev/stderr\n"  # noqa
                   "            exit 1\n"
                   "        else\n"
                   "            echo '{ \"apiVersion\": \"client.authentication.k8s.io/v1beta1\", \"kind\": \"ExecCredential\", \"status\": { \"token\": \"'\"${OS_TOKEN}\"'\"}}'\n"  # noqa
                   "        fi\n"
                   % {'name': cluster.name,
                      'api_address': cluster.api_address,
                      'ca': base64.encode_as_text(certs['ca'])})

    if direct_output:
        return cfg
    if os.path.exists(cfg_file) and not force:
        raise exc.CommandError("File %s exists, aborting." % cfg_file)
    else:
        f = open(cfg_file, "w")
        f.write(cfg)
        f.close()
    if 'csh' in os.environ['SHELL']:
        return "setenv KUBECONFIG %s\n" % cfg_file
    else:
        return "export KUBECONFIG=%s\n" % cfg_file


def _config_cluster_swarm(cluster, cluster_template, cfg_dir,
                          force=False, certs=None):
    """Return and write configuration for the given swarm cluster."""
    tls = "" if cluster_template.tls_disabled else True
    if 'csh' in os.environ['SHELL']:
        result = ("setenv DOCKER_HOST %(docker_host)s\n"
                  "setenv DOCKER_CERT_PATH %(cfg_dir)s\n"
                  "setenv DOCKER_TLS_VERIFY %(tls)s\n"
                  % {'docker_host': cluster.api_address,
                     'cfg_dir': cfg_dir,
                     'tls': tls}
                  )
    else:
        result = ("export DOCKER_HOST=%(docker_host)s\n"
                  "export DOCKER_CERT_PATH=%(cfg_dir)s\n"
                  "export DOCKER_TLS_VERIFY=%(tls)s\n"
                  % {'docker_host': cluster.api_address,
                     'cfg_dir': cfg_dir,
                     'tls': tls}
                  )

    return result


def generate_csr_and_key():
    """Return a dict with a new csr and key."""
    key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend())

    csr = x509.CertificateSigningRequestBuilder().subject_name(
        x509.Name([
            x509.NameAttribute(NameOID.COMMON_NAME, u"admin"),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, u"system:masters")
        ])).sign(key, hashes.SHA256(), default_backend())

    result = {
        'csr': csr.public_bytes(
            encoding=serialization.Encoding.PEM).decode("utf-8"),
        'key': key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()).decode("utf-8"),
    }

    return result



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import testtools

from magnumclient import exceptions
from magnumclient.tests import utils
from magnumclient.v1 import certificates


CERT1 = {
    'cluster_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a53',
    'pem': 'fake-pem'
}
CERT2 = {
    'cluster_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a53',
    'pem': 'fake-pem',
    'csr': 'fake-csr',
}

CREATE_CERT = {'cluster_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a53',
               'csr': 'fake-csr'}

fake_responses = {
    '/v1/certificates':
    {
        'POST': (
            {},
            CERT2,
        )
    },
    '/v1/certificates/%s' % CERT1['cluster_uuid']:
    {
        'GET': (
            {},
            CERT1
        ),
        'PATCH': (
            {},
            None,
        )
    }
}


class CertificateManagerTest(testtools.TestCase):

    def setUp(self):
        super(CertificateManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = certificates.CertificateManager(self.api)

    def test_cert_show_by_id(self):
        cert = self.mgr.get(CERT1['cluster_uuid'])
        expect = [
            ('GET', '/v1/certificates/%s' % CERT1['cluster_uuid'], {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(CERT1['cluster_uuid'], cert.cluster_uuid)
        self.assertEqual(CERT1['pem'], cert.pem)

    def test_cert_create(self):
        cert = self.mgr.create(**CREATE_CERT)
        expect = [
            ('POST', '/v1/certificates', {}, CREATE_CERT),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(CERT2['cluster_uuid'], cert.cluster_uuid)
        self.assertEqual(CERT2['pem'], cert.pem)
        self.assertEqual(CERT2['csr'], cert.csr)

    def test_create_fail(self):
        create_cert_fail = copy.deepcopy(CREATE_CERT)
        create_cert_fail["wrong_key"] = "wrong"
        self.assertRaisesRegex(exceptions.InvalidAttribute,
                               ("Key must be in %s" %
                                ','.join(certificates.CREATION_ATTRIBUTES)),
                               self.mgr.create, **create_cert_fail)
        self.assertEqual([], self.api.calls)

    def test_rotate_ca(self):
        self.mgr.rotate_ca(cluster_uuid=CERT1['cluster_uuid'])
        expect = [
            ('PATCH', '/v1/certificates/%s' % CERT1['cluster_uuid'], {}, None)
        ]
        self.assertEqual(expect, self.api.calls)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from magnumclient.osc.v1 import mservices
from magnumclient.tests.osc.unit.v1 import fakes


class TestServiceList(fakes.TestMagnumClientOSCV1):
    columns = ('id', 'host', 'binary', 'state', 'disabled',
               'disabled_reason', 'created_at', 'updated_at')

    def setUp(self):
        super(TestServiceList, self).setUp()
        self.mservices_mock = self.app.client_manager.container_infra.mservices
        self.mservices_mock.list = mock.Mock()
        fake_service = mock.Mock(
            Binary='magnum-conductor',
            Host='Host1',
            Status='enabled',
            State='up',
            Updated_at=None,
            Disabled_Reason=None,
        )
        fake_service.name = 'test_service'
        self.mservices_mock.list.return_value = [fake_service]

        # Get the command object to test
        self.cmd = mservices.ListService(self.app, None)

    def test_service_list(self):
        arglist = []
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mservices_mock.list.assert_called_with()
        self.assertEqual(self.columns, columns)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from magnumclient.common import cliutils
from magnumclient.tests.v1 import shell_test_base
from magnumclient.v1 import certificates_shell


class ShellTest(shell_test_base.TestCommandLineArgument):

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.get')
    def test_cluster_ca_show_success(self, mock_cert_get, mock_cluster_get):
        mockcluster = mock.MagicMock()
        mockcluster.status = "CREATE_COMPLETE"
        mockcluster.uuid = "xxx"
        mock_cluster_get.return_value = mockcluster
        self._test_arg_success('ca-show xxx')
        expected_args = {}
        expected_args['cluster_uuid'] = mockcluster.uuid
        mock_cert_get.assert_called_once_with(**expected_args)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.get')
    def test_ca_show_failure_duplicate_arg(self, mock_cert_get,
                                           mock_cluster_get):
        self.assertRaises(cliutils.DuplicateArgs,
                          self._test_arg_failure,
                          'ca-show foo --cluster foo',
                          self._duplicate_arg_error)
        mock_cert_get.assert_not_called()
        mock_cluster_get.assert_not_called()

    @mock.patch('os.path.isfile')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.create')
    def test_cluster_ca_sign_success(
        self, mock_cert_create, mock_cluster_get, mock_isfile):
        mock_isfile.return_value = True
        mockcluster = mock.MagicMock()
        mockcluster.status = "CREATE_COMPLETE"
        mockcluster.uuid = "xxx"
        mock_cluster_get.return_value = mockcluster

        fake_csr = 'fake-csr'
        mock_file = mock.mock_open(read_data=fake_csr)
        with mock.patch.object(certificates_shell, 'open', mock_file):
            self._test_arg_success('ca-sign '
                                   '--csr path/csr.pem '
                                   '--cluster xxx')
            expected_args = {}
            expected_args['cluster_uuid'] = mockcluster.uuid
            expected_args['csr'] = fake_csr
            mock_cert_create.assert_called_once_with(**expected_args)

    @mock.patch('os.path.isfile')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.create')
    def test_cluster_ca_sign_with_not_csr(
        self, mock_cert_create, mock_cluster_get, mock_isfile):
        mock_isfile.return_value = False
        mockcluster = mock.MagicMock()
        mockcluster.status = "CREATE_COMPLETE"
        mock_cluster_get.return_value = mockcluster

        fake_csr = 'fake-csr'
        mock_file = mock.mock_open(read_data=fake_csr)
        with mock.patch.object(certificates_shell, 'open', mock_file):
            self._test_arg_success('ca-sign '
                                   '--csr path/csr.pem '
                                   '--cluster xxx')
            mock_isfile.assert_called_once_with('path/csr.pem')
            mock_file.assert_not_called()
            mock_cert_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.get')
    def test_ca_show_failure_with_invalid_field(self, mock_cert_get,
                                                mock_cluster_get):
        self.assertRaises(cliutils.MissingArgs,
                          self._test_arg_failure,
                          'ca-show',
                          self._few_argument_error)
        mock_cert_get.assert_not_called()
        mock_cluster_get.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.rotate_ca')
    def test_ca_rotate(self, mock_rotate_ca, mock_cluster_get):
        mockcluster = mock.MagicMock()
        mockcluster.status = "CREATE_COMPLETE"
        mockcluster.uuid = "xxx"
        mock_cluster_get.return_value = mockcluster
        mock_rotate_ca.return_value = None
        self._test_arg_success('ca-rotate '
                               '--cluster xxx')
        expected_args = {}
        expected_args['cluster_uuid'] = mockcluster.uuid
        mock_rotate_ca.assert_called_once_with(**expected_args)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.rotate_ca')
    def test_ca_rotate_no_cluster_arg(self, mock_rotate_ca, mock_cluster_get):
        _error_msg = [
            (".*(error: argument --cluster is required|"  # py27 compatibility
             "error: the following arguments are required: --cluster).*"),
            ".*Try 'magnum help ca-rotate' for more information.*"
        ]
        self._test_arg_failure('ca-rotate', _error_msg)
        mock_rotate_ca.assert_not_called()
        mock_cluster_get.assert_not_called()



# Copyright (c) 2018 European Organization for Nuclear Research.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import utils
from magnumclient import exceptions
from magnumclient.v1 import baseunit


CREATION_ATTRIBUTES = ['docker_volume_size', 'labels', 'flavor_id', 'image_id',
                       'project_id', 'node_count', 'name', 'role',
                       'min_node_count', 'max_node_count', 'merge_labels']


class NodeGroup(baseunit.BaseTemplate):
    template_name = "NodeGroups"


class NodeGroupManager(baseunit.BaseTemplateManager):
    resource_class = NodeGroup
    template_name = 'nodegroups'
    api_name = 'nodegroups'

    @classmethod
    def _path(cls, cluster_id, id=None):
        path = '/v1/clusters/%s/%s/' % (cluster_id, cls.template_name)
        if id:
            path += str(id)
        return path

    def list(self, cluster_id, limit=None, marker=None, sort_key=None,
             sort_dir=None, role=None, detail=False):
        if limit is not None:
            limit = int(limit)

        filters = utils.common_filters(marker, limit, sort_key, sort_dir)
        path = ''
        if role:
            filters.append('role=%s' % role)
        if detail:
            path += 'detail'
        if filters:
            path += '?' + '&'.join(filters)

        if limit is None:
            return self._list(self._path(cluster_id, id=path),
                              self.__class__.api_name)
        else:
            return self._list_pagination(self._path(cluster_id, id=path),
                                         self.__class__.api_name,
                                         limit=limit)

    def get(self, cluster_id, id):
        try:
            return self._list(self._path(cluster_id, id=id))[0]
        except IndexError:
            return None

    def create(self, cluster_id, **kwargs):
        new = {}
        for (key, value) in kwargs.items():
            if key in CREATION_ATTRIBUTES:
                new[key] = value
            else:
                raise exceptions.InvalidAttribute(
                    "Key must be in %s" % ",".join(CREATION_ATTRIBUTES))
        return self._create(self._path(cluster_id), new)

    def delete(self, cluster_id, id):
        return self._delete(self._path(cluster_id, id=id))

    def update(self, cluster_id, id, patch):
        return self._update(self._path(cluster_id, id=id), patch)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from magnumclient.tests.v1 import shell_test_base


class ShellTest(shell_test_base.TestCommandLineArgument):

    @mock.patch('magnumclient.v1.mservices.MServiceManager.list')
    def test_magnum_service_list_success(self, mock_list):
        self._test_arg_success('service-list')
        mock_list.assert_called_once_with()

    @mock.patch('magnumclient.v1.mservices.MServiceManager.list')
    def test_magnum_service_list_failure(self, mock_list):
        self._test_arg_failure('service-list --wrong',
                               self._unrecognized_arg_error)
        mock_list.assert_not_called()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import base


class Stats(base.Resource):
    def __repr__(self):
        return "<Stats %s>" % self._info


class StatsManager(base.Manager):
    resource_class = Stats

    @staticmethod
    def _path(id=None):
        return '/v1/stats?project_id=%s' % id if id else '/v1/stats'

    def list(self, project_id=None):
        try:
            return self._list(self._path(project_id))[0]
        except IndexError:
            return None



# Copyright (c) 2015 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from magnumclient.v1 import client


def Client(version='1', **kwargs):
    """Factory function to create a new container service client."""
    if version != '1':
        raise ValueError(
            "magnum only has one API version. Valid values for 'version'"
            " are '1'")
    return client.Client(**kwargs)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pbr import version

version_info = version.VersionInfo('python-magnumclient')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib import utils

LOG = logging.getLogger(__name__)

DEFAULT_MAJOR_API_VERSION = '1'
DEFAULT_MAGNUM_API_VERSION = 'latest'
API_VERSION_OPTION = 'os_container_infra_api_version'
API_NAME = 'container_infra'
API_VERSIONS = {
    '1': 'magnumclient.v1.client.Client',
}


def make_client(instance):
    """Returns a magnum client."""
    magnum_client = utils.get_client_class(
        API_NAME,
        instance._api_version[API_NAME],
        API_VERSIONS)
    LOG.debug('Instantiating magnum client: %s', magnum_client)

    client = magnum_client(session=instance.session,
                           region_name=instance._region_name,
                           interface=instance._interface,
                           insecure=instance._insecure,
                           ca_cert=instance._cacert,
                           api_version=DEFAULT_MAGNUM_API_VERSION)
    return client


def build_option_parser(parser):
    """Hook to add global options"""

    parser.add_argument(
        '--os-container-infra-api-version',
        metavar='<container-infra-api-version>',
        default=utils.env(
            'OS_CONTAINER_INFRA_API_VERSION',
            default=DEFAULT_MAJOR_API_VERSION),
        help='Container-Infra API version, default=' +
             DEFAULT_MAJOR_API_VERSION +
             ' (Env: OS_CONTAINER_INFRA_API_VERSION)')
    return parser



# Copyright 2016 EasyStack.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import utils as magnum_utils
from magnumclient.exceptions import InvalidAttribute
from magnumclient.i18n import _

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging


CLUSTER_TEMPLATE_ATTRIBUTES = [
    'insecure_registry',
    'labels',
    'updated_at',
    'floating_ip_enabled',
    'fixed_subnet',
    'master_flavor_id',
    'uuid',
    'no_proxy',
    'https_proxy',
    'tls_disabled',
    'keypair_id',
    'public',
    'http_proxy',
    'docker_volume_size',
    'server_type',
    'external_network_id',
    'cluster_distro',
    'image_id',
    'volume_driver',
    'registry_enabled',
    'docker_storage_driver',
    'apiserver_port',
    'name',
    'created_at',
    'network_driver',
    'fixed_network',
    'coe',
    'flavor_id',
    'master_lb_enabled',
    'dns_nameserver',
    'project_id',
    'hidden',
    'tags',
]


def _show_cluster_template(cluster_template):
    del cluster_template._info['links']
    for field in cluster_template._info:
        if cluster_template._info[field] is None:
            setattr(cluster_template, field, '-')
    columns = CLUSTER_TEMPLATE_ATTRIBUTES
    return columns, osc_utils.get_item_properties(cluster_template, columns)


class CreateClusterTemplate(command.ShowOne):
    """Create a Cluster Template."""
    _description = _("Create a Cluster Template.")

    def get_parser(self, prog_name):
        parser = super(CreateClusterTemplate, self).get_parser(prog_name)

        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the cluster template to create.'))
        parser.add_argument(
            '--coe',
            required=True,
            metavar='<coe>',
            help=_('Specify the Container Orchestration Engine to use.'))
        parser.add_argument(
            '--image',
            required=True,
            metavar='<image>',
            help=_('The name or UUID of the base image to customize for the '
                   'Cluster.'))
        parser.add_argument(
            '--external-network',
            dest='external_network',
            required=True,
            metavar='<external-network>',
            help=_('The external Neutron network name or UUID to connect to '
                   'this Cluster Template.'))
        parser.add_argument(
            '--keypair',
            metavar='<keypair>',
            help=_('The name or UUID of the SSH keypair to load into the '
                   'Cluster nodes.'))
        parser.add_argument(
            '--fixed-network',
            dest='fixed_network',
            metavar='<fixed-network>',
            help=_('The private Neutron network name to connect to this '
                   'Cluster model.'))
        parser.add_argument(
            '--fixed-subnet',
            dest='fixed_subnet',
            metavar='<fixed-subnet>',
            help=_('The private Neutron subnet name to connect to Cluster.'))
        parser.add_argument(
            '--network-driver',
            dest='network_driver',
            metavar='<network-driver>',
            help=_('The network driver name for instantiating container '
                   'networks.'))
        parser.add_argument(
            '--volume-driver',
            dest='volume_driver',
            metavar='<volume-driver>',
            help=_('The volume driver name for instantiating container '
                   'volume.'))
        parser.add_argument(
            '--dns-nameserver',
            dest='dns_nameserver',
            metavar='<dns-nameserver>',
            default='8.8.8.8',
            help=_('The DNS nameserver to use for this cluster template.'))
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            default='m1.medium',
            help=_('The nova flavor name or UUID to use when launching the '
                   'Cluster.'))
        parser.add_argument(
            '--master-flavor',
            dest='master_flavor',
            metavar='<master-flavor>',
            help=_('The nova flavor name or UUID to use when launching the '
                   'master node of the Cluster.'))
        parser.add_argument(
            '--docker-volume-size',
            dest='docker_volume_size',
            metavar='<docker-volume-size>',
            type=int,
            help=_('Specify the number of size in GB for the docker volume '
                   'to use.'))
        parser.add_argument(
            '--docker-storage-driver',
            dest='docker_storage_driver',
            metavar='<docker-storage-driver>',
            default='overlay2',
            help=_('Select a docker storage driver. Supported: devicemapper, '
                   'overlay, overlay2. Default: overlay2'))
        parser.add_argument(
            '--http-proxy',
            dest='http_proxy',
            metavar='<http-proxy>',
            help=_('The http_proxy address to use for nodes in Cluster.'))
        parser.add_argument(
            '--https-proxy',
            dest='https_proxy',
            metavar='<https-proxy>',
            help=_('The https_proxy address to use for nodes in Cluster.'))
        parser.add_argument(
            '--no-proxy',
            dest='no_proxy',
            metavar='<no-proxy>',
            help=_('The no_proxy address to use for nodes in Cluster.'))
        parser.add_argument(
            '--labels',
            metavar='<KEY1=VALUE1,KEY2=VALUE2;KEY3=VALUE3...>',
            action='append',
            default=[],
            help=_('Arbitrary labels in the form of key=value pairs to '
                   'associate with a cluster template. May be used multiple '
                   'times.'))
        parser.add_argument(
            '--tls-disabled',
            dest='tls_disabled',
            action='store_true',
            default=False,
            help=_('Disable TLS in the Cluster.'))
        parser.add_argument(
            '--public',
            action='store_true',
            default=False,
            help=_('Make cluster template public.'))
        parser.add_argument(
            '--registry-enabled',
            dest='registry_enabled',
            action='store_true',
            default=False,
            help=_('Enable docker registry in the Cluster'))
        parser.add_argument(
            '--insecure-registry',
            dest='insecure_registry',
            help=_('The URL pointing to users own private insecure docker'
                   'registry to deploy and run docker containers.'))
        parser.add_argument(
            '--server-type',
            dest='server_type',
            metavar='<server-type>',
            default='vm',
            help=_('Specify the server type to be used for example vm. '
                   'For this release default server type will be vm.'))
        parser.add_argument(
            '--master-lb-enabled',
            dest='master_lb_enabled',
            action='store_true',
            default=False,
            help=_('Indicates whether created Clusters should have a load '
                   'balancer for master nodes or not.'))
        parser.add_argument(
            '--floating-ip-enabled',
            dest='floating_ip_enabled',
            default=[],
            action='append_const',
            const=True,
            help=_('Indicates whether created Clusters should have a '
                   'floating ip.'))
        parser.add_argument(
            '--floating-ip-disabled',
            dest='floating_ip_enabled',
            action='append_const',
            const=False,
            help=_('Disables floating ip creation on the new Cluster'))
        parser.add_argument(
            '--hidden',
            dest='hidden',
            action='store_true',
            default=False,
            help=_('Indicates the cluster template should be hidden.'))
        parser.add_argument(
            '--visible',
            dest='hidden',
            action='store_false',
            help=_('Indicates the cluster template should be visible.'))
        parser.add_argument(
            '--tags',
            action='append',
            default=[],
            metavar='<--tags tag1 --tags tag2,tag3>',
            help=_('Tags to be added to the cluster template.'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        mag_client = self.app.client_manager.container_infra
        args = {
            'name': parsed_args.name,
            'image_id': parsed_args.image,
            'keypair_id': parsed_args.keypair,
            'external_network_id': parsed_args.external_network,
            'coe': parsed_args.coe,
            'fixed_network': parsed_args.fixed_network,
            'fixed_subnet': parsed_args.fixed_subnet,
            'network_driver': parsed_args.network_driver,
            'volume_driver': parsed_args.volume_driver,
            'dns_nameserver': parsed_args.dns_nameserver,
            'flavor_id': parsed_args.flavor,
            'master_flavor_id': parsed_args.master_flavor,
            'docker_volume_size': parsed_args.docker_volume_size,
            'docker_storage_driver': parsed_args.docker_storage_driver,
            'http_proxy': parsed_args.http_proxy,
            'https_proxy': parsed_args.https_proxy,
            'no_proxy': parsed_args.no_proxy,
            'labels': magnum_utils.handle_labels(parsed_args.labels),
            'tls_disabled': parsed_args.tls_disabled,
            'public': parsed_args.public,
            'registry_enabled': parsed_args.registry_enabled,
            'insecure_registry': parsed_args.insecure_registry,
            'server_type': parsed_args.server_type,
            'master_lb_enabled': parsed_args.master_lb_enabled,
        }

        # NOTE: Make new arguments backward compatible
        if parsed_args.hidden:
            args['hidden'] = parsed_args.hidden
        if parsed_args.tags:
            args['tags'] = ','.join(set(
                (','.join(parsed_args.tags)).split(',')))

        if len(parsed_args.floating_ip_enabled) > 1:
            raise InvalidAttribute('--floating-ip-enabled and '
                                   '--floating-ip-disabled are '
                                   'mutually exclusive and '
                                   'should be specified only once.')
        elif len(parsed_args.floating_ip_enabled) == 1:
            args['floating_ip_enabled'] = parsed_args.floating_ip_enabled[0]

        deprecated = ['devicemapper', 'overlay']
        if args['docker_storage_driver'] in deprecated:
            print("WARNING: Docker storage drivers %s are deprecated and will "
                  "be removed in a future release. Use overlay2 instead." %
                  deprecated)

        ct = mag_client.cluster_templates.create(**args)
        print("Request to create cluster template %s accepted"
              % parsed_args.name)
        return _show_cluster_template(ct)


class DeleteClusterTemplate(command.Command):
    """Delete a Cluster Template."""
    _description = _("Delete a Cluster Template.")

    def get_parser(self, prog_name):
        parser = super(DeleteClusterTemplate, self).get_parser(prog_name)
        parser.add_argument(
            'cluster-templates',
            metavar='<cluster-templates>',
            nargs='+',
            help=_('ID or name of the (cluster template)s to delete.'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra

        for cluster_template in getattr(parsed_args, 'cluster-templates'):
            try:
                mag_client.cluster_templates.delete(cluster_template)
                print(
                    "Request to delete cluster template %s has been accepted."
                    % cluster_template)
            except Exception as e:
                print("Delete for cluster template "
                      "%(cluster_template)s failed: %(e)s" %
                      {'cluster_template': cluster_template, 'e': e})


class ListTemplateCluster(command.Lister):
    """List Cluster Templates."""
    _description = _("List Cluster Templates.")

    log = logging.getLogger(__name__ + ".ListTemplateCluster")

    def get_parser(self, prog_name):
        parser = super(ListTemplateCluster, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Maximum number of cluster templates to return'))
        parser.add_argument(
            '--sort-key',
            metavar='<sort-key>',
            help=_('Column to sort results by'))
        parser.add_argument(
            '--sort-dir',
            metavar='<sort-dir>',
            choices=['desc', 'asc'],
            help=_('Direction to sort. "asc" or "desc".'))
        parser.add_argument(
            '--fields',
            default=None,
            metavar='<fields>',
            help=_('Comma-separated list of fields to display. '
                   'Available fields: uuid, name, coe, image_id, public, '
                   'link, apiserver_port, server_type, tls_disabled, '
                   'registry_enabled'
                   )
            )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        columns = ['uuid', 'name', 'tags']
        if parsed_args.fields:
            columns += parsed_args.fields.split(',')
        cts = mag_client.cluster_templates.list(limit=parsed_args.limit,
                                                sort_key=parsed_args.sort_key,
                                                sort_dir=parsed_args.sort_dir)
        return (
            columns,
            (osc_utils.get_item_properties(ct, columns) for ct in cts)
        )


class ShowClusterTemplate(command.ShowOne):
    """Show a Cluster Template."""
    _description = _("Show a Cluster Template.")

    log = logging.getLogger(__name__ + ".ShowClusterTemplate")

    def get_parser(self, prog_name):
        parser = super(ShowClusterTemplate, self).get_parser(prog_name)
        parser.add_argument(
            'cluster-template',
            metavar='<cluster-template>',
            help=_('ID or name of the cluster template to show.'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        ct = getattr(parsed_args, 'cluster-template', None)
        cluster_template = mag_client.cluster_templates.get(ct)

        return _show_cluster_template(cluster_template)


class UpdateClusterTemplate(command.ShowOne):
    """Update a Cluster Template."""

    log = logging.getLogger(__name__ + ".UpdateClusterTemplate")

    def get_parser(self, prog_name):
        parser = super(UpdateClusterTemplate, self).get_parser(prog_name)
        parser.add_argument(
            'cluster-template',
            metavar='<cluster-template>',
            help=_('The name or UUID of cluster template to update'))

        parser.add_argument(
            'op',
            metavar='<op>',
            nargs='?',
            choices=['add', 'replace', 'remove'],
            default=None,
            help=_("Operations: one of 'add', 'replace' or 'remove'"))

        parser.add_argument(
            'attributes',
            metavar='<path=value>',
            nargs='*',
            action='append',
            default=[],
            help=_(
                "Attributes to add/replace or remove (only PATH is necessary "
                "on remove)"))

        parser.set_defaults(hidden=None)
        parser.add_argument(
            '--hidden',
            dest='hidden',
            action='store_true',
            help=_('Hide the cluster template.'))

        parser.add_argument(
            '--visible',
            dest='hidden',
            action='store_false',
            help=_('Make the cluster template visible.'))

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        raw_attrs = parsed_args.attributes[0] if parsed_args.attributes else []
        if parsed_args.op:
            patch = magnum_utils.args_array_to_patch(parsed_args.op, raw_attrs)
        else:
            patch = []
        if parsed_args.hidden is not None:
            patch += [{'op': 'replace', 'path': '/hidden',
                       'value': parsed_args.hidden}]
        if not patch:
            raise InvalidAttribute('At least one of op/attributes or '
                                   '--hidden/--visible must be specified.')
        name = getattr(parsed_args, 'cluster-template', None)
        ct = mag_client.cluster_templates.update(name, patch)
        return _show_cluster_template(ct)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from magnumclient.tests.v1 import shell_test_base


class ShellTest(shell_test_base.TestCommandLineArgument):

    def _get_expected_args_list(self, marker=None, limit=None, sort_dir=None,
                                sort_key=None, all_tenants=False):
        expected_args = {}
        expected_args['marker'] = marker
        expected_args['limit'] = limit
        expected_args['sort_dir'] = sort_dir
        expected_args['sort_key'] = sort_key
        expected_args['all_tenants'] = False
        return expected_args

    def _get_expected_args_create(self, project_id, resource, hard_limit):
        expected_args = {}
        expected_args['project_id'] = project_id
        expected_args['resource'] = resource
        expected_args['hard_limit'] = hard_limit
        return expected_args

    @mock.patch('magnumclient.v1.quotas.QuotasManager.list')
    def test_quotas_list_success(self, mock_list):
        self._test_arg_success('quotas-list')
        expected_args = self._get_expected_args_list()
        mock_list.assert_called_once_with(**expected_args)

    @mock.patch('magnumclient.v1.quotas.QuotasManager.list')
    def test_quotas_list_failure(self, mock_list):
        self._test_arg_failure('quotas-list --wrong',
                               self._unrecognized_arg_error)
        mock_list.assert_not_called()

    @mock.patch('magnumclient.v1.quotas.QuotasManager.create')
    def test_quotas_create_success(self, mock_create):
        self._test_arg_success('quotas-create --project-id abc '
                               '--resource Cluster '
                               '--hard-limit 15')
        expected_args = self._get_expected_args_create('abc', 'Cluster', 15)
        mock_create.assert_called_with(**expected_args)

    @mock.patch('magnumclient.v1.quotas.QuotasManager.create')
    def test_quotas_create_failure_only_project_id(self, mock_create):
        self._test_arg_failure('quotas-create --project-id abc',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.quotas.QuotasManager.create')
    def test_quotas_create_failure_only_resource(self, mock_create):
        self._test_arg_failure('quotas-create --resource Cluster',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.quotas.QuotasManager.create')
    def test_quotas_create_failure_only_hard_limit(self, mock_create):
        self._test_arg_failure('quotas-create --hard-limit 10',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.quotas.QuotasManager.create')
    def test_quotas_create_failure_no_arg(self, mock_create):
        self._test_arg_failure('quotas-create',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.quotas.QuotasManager.delete')
    def test_quotas_delete_success(self, mock_delete):
        self._test_arg_success(
            'quotas-delete --project-id xxx --resource Cluster')
        mock_delete.assert_called_once_with('xxx', 'Cluster')

    @mock.patch('magnumclient.v1.quotas.QuotasManager.delete')
    def test_quotas_delete_failure_no_project_id(self, mock_delete):
        self._test_arg_failure('quotas-delete --resource Cluster',
                               self._mandatory_arg_error)
        mock_delete.assert_not_called()

    @mock.patch('magnumclient.v1.quotas.QuotasManager.delete')
    def test_quotas_delete_failure_no_resource(self, mock_delete):
        self._test_arg_failure('quotas-delete --project-id xxx',
                               self._mandatory_arg_error)
        mock_delete.assert_not_called()

    @mock.patch('magnumclient.v1.quotas.QuotasManager.get')
    def test_quotas_show_success(self, mock_show):
        self._test_arg_success('quotas-show --project-id abc '
                               '--resource Cluster')
        mock_show.assert_called_once_with('abc', 'Cluster')

    @mock.patch('magnumclient.v1.quotas.QuotasManager.get')
    def test_quotas_show_failure_no_arg(self, mock_show):
        self._test_arg_failure('quotas-show',
                               self._mandatory_arg_error)
        mock_show.assert_not_called()

    @mock.patch('magnumclient.v1.quotas.QuotasManager.update')
    def test_quotas_update_success(self, mock_update):
        self._test_arg_success('quotas-update --project-id abc '
                               '--resource Cluster '
                               '--hard-limit 20')
        patch = {'project_id': 'abc', 'resource': 'Cluster', 'hard_limit': 20}
        mock_update.assert_called_once_with('abc', 'Cluster', patch)



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common.apiclient import exceptions
from magnumclient.common.apiclient.exceptions import *  # noqa


# NOTE(akurilin): This alias is left here since v.0.1.3 to support backwards
# compatibility.
InvalidEndpoint = exceptions.EndpointException
CommunicationError = exceptions.ConnectionRefused
HTTPBadRequest = exceptions.BadRequest
HTTPInternalServerError = exceptions.InternalServerError
HTTPNotFound = exceptions.NotFound
HTTPServiceUnavailable = exceptions.ServiceUnavailable


class AmbiguousAuthSystem(exceptions.ClientException):
    """Could not obtain token and endpoint using provided credentials."""
    pass


# Alias for backwards compatibility
AmbigiousAuthSystem = AmbiguousAuthSystem


class InvalidAttribute(exceptions.ClientException):
    pass


def from_response(response, message=None, traceback=None, method=None,
                  url=None):
    """Return an HttpError instance based on response from httplib/requests."""

    error_body = {}
    if message:
        error_body['message'] = message
    if traceback:
        error_body['details'] = traceback

    if hasattr(response, 'status') and not hasattr(response, 'status_code'):
        # NOTE(akurilin): These modifications around response object give
        # ability to get all necessary information in method `from_response`
        # from common code, which expecting response object from `requests`
        # library instead of object from `httplib/httplib2` library.
        response.status_code = response.status
        response.headers = {
            'Content-Type': response.getheader('content-type', "")}

    if hasattr(response, 'status_code'):
        # NOTE(hongbin): This allows SessionClient to handle faultstring.
        response.json = lambda: {'error': error_body}

    if (response.headers.get('Content-Type', '').startswith('text/') and
            not hasattr(response, 'text')):
        # NOTE(clif_h): There seems to be a case in the
        # common.apiclient.exceptions module where if the
        # content-type of the response is text/* then it expects
        # the response to have a 'text' attribute, but that
        # doesn't always seem to necessarily be the case.
        # This is to work around that problem.
        response.text = ''

    return exceptions.from_response(response, method, url)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils
from magnumclient.i18n import _
from osc_lib.command import command

QUOTA_ATTRIBUTES = [
    'resource',
    'created_at',
    'updated_at',
    'hard_limit',
    'project_id',
    'id'
]


def _show_quota(quota):
    utils.print_dict(quota._info)


class CreateQuotas(command.Command):
    _description = _("Create a quota.")

    def get_parser(self, prog_name):
        parser = super(CreateQuotas, self).get_parser(prog_name)
        parser.add_argument('--project-id',
                            required=True,
                            metavar='<project-id>',
                            help='Project ID')
        parser.add_argument('--resource',
                            required=True,
                            metavar='<resource>',
                            help='Resource name.')
        parser.add_argument('--hard-limit',
                            metavar='<hard-limit>',
                            type=int,
                            default=1,
                            help='Max resource limit (default: hard-limit=1)')
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra

        opts = {
            'project_id': parsed_args.project_id,
            'resource': parsed_args.resource,
            'hard_limit': parsed_args.hard_limit
        }
        try:
            quota = mag_client.quotas.create(**opts)
            _show_quota(quota)
        except Exception as e:
            print("Create quota for project_id %(id)s resource %(res)s failed:"
                  " %(e)s" % {'id': parsed_args.project_id,
                              'res': parsed_args.resource,
                              'e': e})


class DeleteQuotas(command.Command):
    _description = _("Delete specified resource quota.")

    def get_parser(self, prog_name):
        parser = super(DeleteQuotas, self).get_parser(prog_name)
        parser.add_argument('--project-id',
                            required=True,
                            metavar='<project-id>',
                            help='Project ID')
        parser.add_argument('--resource',
                            required=True,
                            metavar='<resource>',
                            help='Resource name.')
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra

        try:
            mag_client.quotas.delete(parsed_args.project_id,
                                     parsed_args.resource)
            print("Request to delete quota for project id %(id)s and resource "
                  "%(res)s has been accepted." % {'id': parsed_args.project_id,
                                                  'res': parsed_args.resource})
        except Exception as e:
            print("Quota delete failed for project id %(id)s and resource "
                  "%(res)s :%(e)s" % {'id': parsed_args.project_id,
                                      'res': parsed_args.resource,
                                      'e': e})


class ShowQuotas(command.Command):
    _description = _("Show details about the given project resource quota.")

    def get_parser(self, prog_name):
        parser = super(ShowQuotas, self).get_parser(prog_name)
        parser.add_argument('--project-id',
                            required=True,
                            metavar='<project-id>',
                            help='Project ID')
        parser.add_argument('--resource',
                            required=True,
                            metavar='<resource>',
                            help='Resource name.')
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        project_id = parsed_args.project_id
        resource = parsed_args.resource
        quota = mag_client.quotas.get(project_id, resource)
        _show_quota(quota)


class UpdateQuotas(command.Command):
    _description = _("Update information about the given "
                     "project resource quota.")

    def get_parser(self, prog_name):
        parser = super(UpdateQuotas, self).get_parser(prog_name)
        parser.add_argument('--project-id',
                            required=True,
                            metavar='<project-id>',
                            help='Project ID')
        parser.add_argument('--resource',
                            required=True,
                            metavar='<resource>',
                            help='Resource name.')
        parser.add_argument('--hard-limit',
                            metavar='<hard-limit>',
                            type=int,
                            default=1,
                            help='Max resource limit (default: hard-limit=1)')
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra

        opts = {
            'project_id': parsed_args.project_id,
            'resource': parsed_args.resource,
            'hard_limit': parsed_args.hard_limit
        }
        try:
            quota = mag_client.quotas.update(parsed_args.project_id,
                                             parsed_args.resource, opts)
            _show_quota(quota)
        except Exception as e:
            print("Update quota for project_id %(id)s resource %(res)s failed:"
                  " %(e)s" % {'id': parsed_args.project_id,
                              'res': parsed_args.resource,
                              'e': e})


class ListQuotas(command.Command):
    _description = _("Print a list of available quotas.")

    def get_parser(self, prog_name):
        parser = super(ListQuotas, self).get_parser(prog_name)
        parser.add_argument('--marker',
                            metavar='<marker>',
                            default=None,
                            help=_('The last quota UUID of the previous page; '
                                   'displays list of quotas after "marker".'))
        parser.add_argument('--limit',
                            metavar='<limit>',
                            type=int,
                            help='Maximum number of quotas to return.')
        parser.add_argument('--sort-key',
                            metavar='<sort-key>',
                            help='Column to sort results by.')
        parser.add_argument('--sort-dir',
                            metavar='<sort-dir>',
                            choices=['desc', 'asc'],
                            help='Direction to sort. "asc" or "desc".')
        parser.add_argument('--all-tenants',
                            action='store_true',
                            default=False,
                            help='Flag to indicate list all tenant quotas.')
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra

        quotas = mag_client.quotas.list(marker=parsed_args.marker,
                                        limit=parsed_args.limit,
                                        sort_key=parsed_args.sort_key,
                                        sort_dir=parsed_args.sort_dir,
                                        all_tenants=parsed_args.all_tenants)
        columns = ['project_id', 'resource', 'hard_limit']
        utils.print_list(quotas, columns,
                         {'versions':
                          magnum_utils.print_list_field('versions')},
                         sortby_index=None)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import utils
from magnumclient import exceptions
from magnumclient.v1 import basemodels


CREATION_ATTRIBUTES = ['project_id', 'resource', 'hard_limit']


class Quotas(basemodels.BaseModel):
    model_name = "Quotas"


class QuotasManager(basemodels.BaseModelManager):
    api_name = "quotas"
    resource_class = Quotas

    @staticmethod
    def _path(id=None, resource=None):
        if not id:
            return '/v1/quotas'

        return '/v1/quotas/%(id)s/%(res)s' % {'id': id, 'res': resource}

    def list(self, limit=None, marker=None, sort_key=None,
             sort_dir=None, all_tenants=False):

        if limit is not None:
            limit = int(limit)

        filters = utils.common_filters(marker, limit, sort_key, sort_dir)

        if all_tenants:
            filters.append('all_tenants=True')

        path = self._path()
        if filters:
            path += '?' + '&'.join(filters)

        if limit is None:
            return self._list(path, self.api_name)
        else:
            return self._list_pagination(path, self.api_name,
                                         limit=limit)

    def get(self, id, resource):
        try:
            return self._list(self._path(id, resource))[0]
        except IndexError:
            return None

    def create(self, **kwargs):
        new = {}
        for (key, value) in kwargs.items():
            if key in CREATION_ATTRIBUTES:
                new[key] = value
            else:
                raise exceptions.InvalidAttribute(
                    "Key must be in %s" % ",".join(CREATION_ATTRIBUTES))
        return self._create(self._path(), new)

    def delete(self, id, resource):
        return self._delete(self._path(id, resource))

    def update(self, id, resource, patch):
        url = self._path(id, resource)
        return self._update(url, patch)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from magnumclient.common import cliutils
from magnumclient import exceptions
from magnumclient.tests.v1 import shell_test_base
from magnumclient.tests.v1 import test_clustertemplates_shell
from magnumclient.v1.clusters import Cluster


class FakeCluster(Cluster):
    def __init__(self, manager=None, info={}, **kwargs):
        Cluster.__init__(self, manager=manager, info=info)
        self.uuid = kwargs.get('uuid', 'x')
        self.keypair = kwargs.get('keypair', 'x')
        self.docker_volume_size = kwargs.get('docker_volume_size', 3)
        self.labels = kwargs.get('labels', 'key=val')
        self.name = kwargs.get('name', 'x')
        self.cluster_template_id = kwargs.get('cluster_template_id', 'x')
        self.stack_id = kwargs.get('stack_id', 'x')
        self.status = kwargs.get('status', 'x')
        self.master_count = kwargs.get('master_count', 1)
        self.node_count = kwargs.get('node_count', 1)
        self.links = kwargs.get('links', [])
        self.create_timeout = kwargs.get('create_timeout', 60)


class FakeCert(object):
    def __init__(self, pem):
        self.pem = pem


class ShellTest(shell_test_base.TestCommandLineArgument):

    def _get_expected_args_list(self, marker=None, limit=None,
                                sort_dir=None, sort_key=None):
        expected_args = {}
        expected_args['marker'] = marker
        expected_args['limit'] = limit
        expected_args['sort_dir'] = sort_dir
        expected_args['sort_key'] = sort_key

        return expected_args

    def _get_expected_args_create(self, cluster_template_id, name=None,
                                  master_count=1, node_count=1,
                                  create_timeout=60, keypair=None,
                                  docker_volume_size=None,
                                  labels=None,
                                  discovery_url=None):
        expected_args = {}
        expected_args['name'] = name
        expected_args['cluster_template_id'] = cluster_template_id
        expected_args['master_count'] = master_count
        expected_args['node_count'] = node_count
        expected_args['create_timeout'] = create_timeout
        expected_args['discovery_url'] = discovery_url
        expected_args['keypair'] = keypair
        if docker_volume_size is not None:
            expected_args['docker_volume_size'] = docker_volume_size
        if labels is not None:
            expected_args['labels'] = labels

        return expected_args

    @mock.patch('magnumclient.v1.clusters.ClusterManager.list')
    def test_cluster_list_success(self, mock_list):
        self._test_arg_success('cluster-list')
        expected_args = self._get_expected_args_list()
        mock_list.assert_called_once_with(**expected_args)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.list')
    def test_cluster_list_success_with_arg(self, mock_list):
        self._test_arg_success('cluster-list '
                               '--marker some_uuid '
                               '--limit 1 '
                               '--sort-dir asc '
                               '--sort-key uuid')
        expected_args = self._get_expected_args_list('some_uuid', 1,
                                                     'asc', 'uuid')
        mock_list.assert_called_once_with(**expected_args)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.list')
    def test_cluster_list_ignored_duplicated_field(self, mock_list):
        mock_list.return_value = [FakeCluster()]
        self._test_arg_success(
            'cluster-list --fields status,status,status,name',
            keyword=('\n| uuid | name | keypair | node_count | master_count | '
                     'status |\n'))
        expected_args = self._get_expected_args_list()
        mock_list.assert_called_once_with(**expected_args)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.list')
    def test_cluster_list_failure_with_invalid_field(self, mock_list):
        mock_list.return_value = [FakeCluster()]
        _error_msg = [".*?^Non-existent fields are specified: ['xxx','zzz']"]
        self.assertRaises(exceptions.CommandError,
                          self._test_arg_failure,
                          'cluster-list --fields xxx,stack_id,zzz,status',
                          _error_msg)
        expected_args = self._get_expected_args_list()
        mock_list.assert_called_once_with(**expected_args)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.list')
    def test_cluster_list_failure_invalid_arg(self, mock_list):
        _error_msg = [
            '.*?^usage: magnum cluster-list ',
            '.*?^error: argument --sort-dir: invalid choice: ',
            ".*?^Try 'magnum help cluster-list' for more information."
            ]
        self._test_arg_failure('cluster-list --sort-dir aaa', _error_msg)
        mock_list.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.list')
    def test_cluster_list_failure(self, mock_list):
        self._test_arg_failure('cluster-list --wrong',
                               self._unrecognized_arg_error)
        mock_list.assert_not_called()

    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_success(self, mock_create, mock_get):
        mock_ct = mock.MagicMock()
        mock_ct.uuid = 'xxx'
        mock_get.return_value = mock_ct
        self._test_arg_success('cluster-create test '
                               '--cluster-template xxx '
                               '--node-count 123 --timeout 15')
        expected_args = self._get_expected_args_create('xxx', name='test',
                                                       node_count=123,
                                                       create_timeout=15)
        mock_create.assert_called_with(**expected_args)

        self._test_arg_success('cluster-create --cluster-template xxx')
        expected_args = self._get_expected_args_create('xxx')
        mock_create.assert_called_with(**expected_args)

        self._test_arg_success('cluster-create --cluster-template xxx '
                               '--keypair x')
        expected_args = self._get_expected_args_create('xxx',
                                                       keypair='x')
        mock_create.assert_called_with(**expected_args)

        self._test_arg_success('cluster-create --cluster-template xxx '
                               '--docker-volume-size 20')
        expected_args = self._get_expected_args_create('xxx',
                                                       docker_volume_size=20)

        self._test_arg_success('cluster-create --cluster-template xxx '
                               '--labels key=val')
        expected_args = self._get_expected_args_create('xxx',
                                                       labels={'key': 'val'})
        mock_create.assert_called_with(**expected_args)

        self._test_arg_success('cluster-create test '
                               '--cluster-template xxx')
        expected_args = self._get_expected_args_create('xxx',
                                                       name='test')
        mock_create.assert_called_with(**expected_args)

        self._test_arg_success('cluster-create --cluster-template xxx '
                               '--node-count 123')
        expected_args = self._get_expected_args_create('xxx',
                                                       node_count=123)
        mock_create.assert_called_with(**expected_args)

        self._test_arg_success('cluster-create --cluster-template xxx '
                               '--node-count 123 --master-count 123')
        expected_args = self._get_expected_args_create('xxx',
                                                       master_count=123,
                                                       node_count=123)
        mock_create.assert_called_with(**expected_args)

        self._test_arg_success('cluster-create --cluster-template xxx '
                               '--timeout 15')
        expected_args = self._get_expected_args_create('xxx',
                                                       create_timeout=15)
        mock_create.assert_called_with(**expected_args)

    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_deprecation_warnings(self, mock_create,
                                                 mock_get):
        self._test_arg_failure('cluster-create --cluster-template xxx '
                               '--keypair-id x',
                               self._deprecated_warning)
        self.assertTrue(mock_create.called)
        self.assertTrue(mock_get.called)

        self._test_arg_failure('cluster-create --cluster-template xxx '
                               '--name foo ',
                               self._deprecated_warning)
        self.assertTrue(mock_create.called)
        self.assertTrue(mock_get.called)

    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_deprecation_errors(self,
                                               mock_create,
                                               mock_get):
        self._test_arg_failure('cluster-create --cluster-template xxx '
                               '--keypair-id x --keypair x',
                               self._too_many_group_arg_error)
        self.assertFalse(mock_create.called)
        self.assertFalse(mock_get.called)

    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    def test_cluster_show_clustertemplate_metadata(self,
                                                   mock_cluster,
                                                   mock_clustertemplate):
        mock_cluster.return_value = mock.MagicMock(cluster_template_id=0)

        mock_clustertemplate.return_value = \
            test_clustertemplates_shell.FakeClusterTemplate(info={'links': 0,
                                                                  'uuid': 0,
                                                                  'id': 0,
                                                                  'name': ''})

        self._test_arg_success('cluster-show --long x')
        mock_cluster.assert_called_once_with('x')
        mock_clustertemplate.assert_called_once_with(0)

    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def _test_cluster_create_success(self, cmd, expected_args, expected_kwargs,
                                     mock_create, mock_get):
        mock_ct = mock.MagicMock()
        mock_ct.uuid = 'xxx'
        mock_get.return_value = mock_ct
        self._test_arg_success(cmd)
        expected = self._get_expected_args_create(*expected_args,
                                                  **expected_kwargs)
        mock_create.assert_called_with(**expected)

    def test_cluster_create_success_only_clustertemplate_arg(self):
        self._test_cluster_create_success(
            'cluster-create --cluster-template xxx',
            ['xxx'],
            {})

    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_success_only_positional_name(self,
                                                         mock_create,
                                                         mock_get):
        self._test_cluster_create_success(
            'cluster-create foo --cluster-template xxx',
            ['xxx'],
            {'name': 'foo'})

    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_success_only_optional_name(self,
                                                       mock_create,
                                                       mock_get):
        self._test_cluster_create_success(
            'cluster-create --name foo --cluster-template xxx',
            ['xxx'],
            {'name': 'foo'})

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_only_name(self, mock_create):
        self._test_arg_failure('cluster-create --name test',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_only_keypair(self, mock_create):
        self._test_arg_failure('cluster-create --keypair test',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_only_docker_volume_size(self, mock_create):
        self._test_arg_failure('cluster-create --docker_volume_size 20',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_only_labels(self, mock_create):
        self._test_arg_failure('cluster-create --labels key=val',
                               self._mandatory_arg_error)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_only_node_count(self, mock_create):
        self._test_arg_failure('cluster-create --node-count 1',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_invalid_node_count(self, mock_create):
        self._test_arg_failure('cluster-create --cluster-template xxx '
                               '--node-count test',
                               self._invalid_value_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_only_cluster_create_timeout(self,
                                                                mock_create):
        self._test_arg_failure('cluster-create --timeout 15',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_no_arg(self, mock_create):
        self._test_arg_failure('cluster-create',
                               self._mandatory_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_invalid_master_count(self, mock_create):
        self._test_arg_failure('cluster-create --cluster-template xxx '
                               '--master-count test',
                               self._invalid_value_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.create')
    def test_cluster_create_failure_duplicate_name(self, mock_create):
        self.assertRaises(cliutils.DuplicateArgs,
                          self._test_arg_failure,
                          'cluster-create foo --name bar '
                          '--cluster-template xxx',
                          self._duplicate_arg_error)
        mock_create.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.delete')
    def test_cluster_delete_success(self, mock_delete):
        self._test_arg_success('cluster-delete xxx')
        mock_delete.assert_called_once_with('xxx')

    @mock.patch('magnumclient.v1.clusters.ClusterManager.delete')
    def test_cluster_delete_multiple_id_success(self, mock_delete):
        self._test_arg_success('cluster-delete xxx xyz')
        calls = [mock.call('xxx'), mock.call('xyz')]
        mock_delete.assert_has_calls(calls)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.delete')
    def test_cluster_delete_failure_no_arg(self, mock_delete):
        self._test_arg_failure('cluster-delete', self._few_argument_error)
        mock_delete.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    def test_cluster_show_success(self, mock_show):
        self._test_arg_success('cluster-show xxx')
        mock_show.assert_called_once_with('xxx')

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    def test_cluster_show_failure_no_arg(self, mock_show):
        self._test_arg_failure('cluster-show', self._few_argument_error)
        mock_show.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.update')
    def test_cluster_update_success(self, mock_update):
        self._test_arg_success('cluster-update test add test=test')
        patch = [{'op': 'add', 'path': '/test', 'value': 'test'}]
        mock_update.assert_called_once_with('test', patch, False)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.update')
    def test_cluster_update_success_many_attribute(self, mock_update):
        self._test_arg_success('cluster-update test add test=test test1=test1')
        patch = [{'op': 'add', 'path': '/test', 'value': 'test'},
                 {'op': 'add', 'path': '/test1', 'value': 'test1'}]
        mock_update.assert_called_once_with('test', patch, False)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.update')
    def test_cluster_update_success_rollback(self, mock_update):
        self._test_arg_success('cluster-update test add test=test --rollback')
        patch = [{'op': 'add', 'path': '/test', 'value': 'test'}]
        mock_update.assert_called_once_with('test', patch, True)

    @mock.patch('magnumclient.v1.clusters.ClusterManager.update')
    def test_cluster_update_rollback_old_api_version(self, mock_update):
        self.assertRaises(
            exceptions.CommandError,
            self.shell,
            '--magnum-api-version 1.2 cluster-update '
            'test add test=test --rollback')
        mock_update.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.update')
    def test_cluster_update_failure_wrong_op(self, mock_update):
        _error_msg = [
            '.*?^usage: magnum cluster-update ',
            '.*?^error: argument <op>: invalid choice: ',
            ".*?^Try 'magnum help cluster-update' for more information."
            ]
        self._test_arg_failure('cluster-update test wrong test=test',
                               _error_msg)
        mock_update.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.update')
    def test_cluster_update_failure_wrong_attribute(self, mock_update):
        _error_msg = [
            '.*?^ERROR: Attributes must be a list of PATH=VALUE'
            ]
        self.assertRaises(exceptions.CommandError, self._test_arg_failure,
                          'cluster-update test add test', _error_msg)
        mock_update.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.update')
    def test_cluster_update_failure_few_args(self, mock_update):
        _error_msg = [
            '.*?^usage: magnum cluster-update ',
            '.*?^error: (the following arguments|too few arguments)',
            ".*?^Try 'magnum help cluster-update' for more information."
            ]
        self._test_arg_failure('cluster-update', _error_msg)
        mock_update.assert_not_called()

        self._test_arg_failure('cluster-update test', _error_msg)
        mock_update.assert_not_called()

        self._test_arg_failure('cluster-update test add', _error_msg)
        mock_update.assert_not_called()

    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    def test_cluster_config_success(self, mock_cluster, mock_clustertemplate):
        mock_cluster.return_value = FakeCluster(status='UPDATE_COMPLETE')
        self._test_arg_success('cluster-config xxx')
        mock_cluster.assert_called_with('xxx')

        mock_cluster.return_value = FakeCluster(status='CREATE_COMPLETE')
        self._test_arg_success('cluster-config xxx')
        mock_cluster.assert_called_with('xxx')

        self._test_arg_success('cluster-config --dir /tmp xxx')
        mock_cluster.assert_called_with('xxx')

        self._test_arg_success('cluster-config --force xxx')
        mock_cluster.assert_called_with('xxx')

        self._test_arg_success('cluster-config --dir /tmp --force xxx')
        mock_cluster.assert_called_with('xxx')

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    def test_cluster_config_failure_no_arg(self, mock_cluster):
        self._test_arg_failure('cluster-config', self._few_argument_error)
        mock_cluster.assert_not_called()

    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    def test_cluster_config_failure_wrong_arg(self, mock_cluster):
        self._test_arg_failure('cluster-config xxx yyy',
                               self._unrecognized_arg_error)
        mock_cluster.assert_not_called()

    @mock.patch('os.path.exists')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.create')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.get')
    @mock.patch('magnumclient.v1.cluster_templates.ClusterTemplateManager.get')
    @mock.patch('magnumclient.v1.clusters.ClusterManager.get')
    def _test_cluster_config_success(self, mock_cluster, mock_ct,
                                     mock_cert_get, mock_cert_create,
                                     mock_exists, coe, shell, tls_disable):
        cert = FakeCert(pem='foo bar')
        mock_exists.return_value = False
        mock_cluster.return_value = FakeCluster(status='CREATE_COMPLETE',
                                                info={
                                                    'name': 'Kluster',
                                                    'api_address': '10.0.0.1'},
                                                cluster_template_id='fake_ct',
                                                uuid='fake_cluster')
        mock_cert_get.return_value = cert
        mock_cert_create.return_value = cert
        mock_ct.return_value = test_clustertemplates_shell.\
            FakeClusterTemplate(coe=coe, name='fake_ct',
                                tls_disabled=tls_disable)
        with mock.patch.dict('os.environ', {'SHELL': shell}):
            self._test_arg_success('cluster-config test_cluster')

        self.assertTrue(mock_exists.called)
        mock_cluster.assert_called_once_with('test_cluster')
        mock_ct.assert_called_once_with('fake_ct')
        if not tls_disable:
            mock_cert_create.assert_called_once_with(
                cluster_uuid='fake_cluster', csr=mock.ANY)
            mock_cert_get.assert_called_once_with(cluster_uuid='fake_cluster')

    def test_cluster_config_swarm_success_with_tls_csh(self):
        self._test_cluster_config_success(coe='swarm', shell='csh',
                                          tls_disable=False)

    def test_cluster_config_swarm_success_with_tls_non_csh(self):
        self._test_cluster_config_success(coe='swarm', shell='zsh',
                                          tls_disable=False)

    def test_cluster_config_swarm_success_without_tls_csh(self):
        self._test_cluster_config_success(coe='swarm', shell='csh',
                                          tls_disable=True)

    def test_cluster_config_swarm_success_without_tls_non_csh(self):
        self._test_cluster_config_success(coe='swarm', shell='zsh',
                                          tls_disable=True)

    def test_cluster_config_k8s_success_with_tls_csh(self):
        self._test_cluster_config_success(coe='kubernetes', shell='csh',
                                          tls_disable=False)

    def test_cluster_config_k8s_success_with_tls_non_csh(self):
        self._test_cluster_config_success(coe='kubernetes', shell='zsh',
                                          tls_disable=False)

    def test_cluster_config_k8s_success_without_tls_csh(self):
        self._test_cluster_config_success(coe='kubernetes', shell='csh',
                                          tls_disable=True)

    def test_cluster_config_k8s_success_without_tls_non_csh(self):
        self._test_cluster_config_success(coe='kubernetes', shell='zsh',
                                          tls_disable=True)



#   Copyright 2016 Easystack. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
from unittest import mock
from unittest.mock import call

from magnumclient.exceptions import InvalidAttribute
from magnumclient.osc.v1 import cluster_templates as osc_ct
from magnumclient.tests.osc.unit.v1 import fakes as magnum_fakes

from osc_lib import exceptions as osc_exceptions


class TestClusterTemplate(magnum_fakes.TestMagnumClientOSCV1):
    default_create_args = {
        'coe': 'kubernetes',
        'dns_nameserver': '8.8.8.8',
        'docker_storage_driver': 'overlay2',
        'docker_volume_size': None,
        'external_network_id': 'public',
        'fixed_network': None,
        'fixed_subnet': None,
        'flavor_id': 'm1.medium',
        'http_proxy': None,
        'https_proxy': None,
        'image_id': 'fedora-atomic-latest',
        'keypair_id': None,
        'labels': {},
        'master_flavor_id': None,
        'master_lb_enabled': False,
        'name': 'fake-ct-1',
        'network_driver': None,
        'no_proxy': None,
        'public': False,
        'registry_enabled': False,
        'insecure_registry': None,
        'server_type': 'vm',
        'tls_disabled': False,
        'volume_driver': None,
    }

    def setUp(self):
        super(TestClusterTemplate, self).setUp()

        self.cluster_templates_mock = (
            self.app.client_manager.container_infra.cluster_templates)


class TestClusterTemplateCreate(TestClusterTemplate):

    def setUp(self):
        super(TestClusterTemplateCreate, self).setUp()

        attr = dict()
        attr['name'] = 'fake-ct-1'
        self.new_ct = (
            magnum_fakes.FakeClusterTemplate.create_one_cluster_template(attr))

        self.cluster_templates_mock.create = mock.Mock()
        self.cluster_templates_mock.create.return_value = self.new_ct

        self.cluster_templates_mock.get = mock.Mock()
        self.cluster_templates_mock.get.return_value = copy.deepcopy(
            self.new_ct)

        self.cluster_templates_mock.update = mock.Mock()
        self.cluster_templates_mock.update.return_value = self.new_ct

        # Get the command object to test
        self.cmd = osc_ct.CreateClusterTemplate(self.app, None)

        self.data = tuple(map(lambda x: getattr(self.new_ct, x),
                              osc_ct.CLUSTER_TEMPLATE_ATTRIBUTES))

    def test_cluster_template_create_required_args_pass(self):
        """Verifies required arguments."""

        arglist = [
            '--coe', self.new_ct.coe,
            '--external-network', self.new_ct.external_network_id,
            '--image', self.new_ct.image_id,
            self.new_ct.name
        ]
        verifylist = [
            ('coe', self.new_ct.coe),
            ('external_network', self.new_ct.external_network_id),
            ('image', self.new_ct.image_id),
            ('name', self.new_ct.name)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.create.assert_called_with(
            **self.default_create_args)

    def test_cluster_template_create_missing_required_arg(self):
        """Verifies missing required arguments."""

        arglist = [
            '--external-network', self.new_ct.external_network_id,
            '--image', self.new_ct.image_id
        ]
        verifylist = [
            ('external_network', self.new_ct.external_network_id),
            ('image', self.new_ct.image_id)
        ]
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)
        # Verify all required args are checked and not just coe
        arglist.append('--coe')
        arglist.append(self.new_ct.coe)
        verifylist.append(('coe', self.new_ct.coe))
        arglist.remove('--image')
        arglist.remove(self.new_ct.image_id)
        verifylist.remove(('image', self.new_ct.image_id))
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)

        arglist.remove('--external-network')
        arglist.remove(self.new_ct.external_network_id)
        verifylist.remove(
            ('external_network', self.new_ct.external_network_id))
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_cluster_template_create_floating_ips(self):
        """Verifies floating ip parameters."""
        arglist = [
            '--coe', self.new_ct.coe,
            '--external-network', self.new_ct.external_network_id,
            '--image', self.new_ct.image_id,
            '--floating-ip-enabled',
            self.new_ct.name
        ]
        verifylist = [
            ('coe', self.new_ct.coe),
            ('external_network', self.new_ct.external_network_id),
            ('image', self.new_ct.image_id),
            ('floating_ip_enabled', [True]),
            ('name', self.new_ct.name)
        ]
        self.default_create_args['floating_ip_enabled'] = True
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.default_create_args.pop('floating_ip_enabled')

        arglist.append('--floating-ip-disabled')
        verifylist.remove(('floating_ip_enabled', [True]))
        verifylist.append(('floating_ip_enabled', [True, False]))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(InvalidAttribute, self.cmd.take_action, parsed_args)


class TestClusterTemplateDelete(TestClusterTemplate):

    def setUp(self):
        super(TestClusterTemplateDelete, self).setUp()

        self.cluster_templates_mock.delete = mock.Mock()
        self.cluster_templates_mock.delete.return_value = None

        # Get the command object to test
        self.cmd = osc_ct.DeleteClusterTemplate(self.app, None)

    def test_cluster_template_delete_one(self):
        arglist = ['foo']
        verifylist = [('cluster-templates', ['foo'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.delete.assert_called_with('foo')

    def test_cluster_template_delete_multiple(self):
        arglist = ['foo', 'bar']
        verifylist = [('cluster-templates', ['foo', 'bar'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.delete.assert_has_calls([call('foo'),
                                                            call('bar')])

    def test_cluster_template_delete_bad_uuid(self):
        self.cluster_templates_mock.delete.side_effect = (
            osc_exceptions.NotFound(404))
        arglist = ['foo']
        verifylist = [('cluster-templates', ['foo'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        returns = self.cmd.take_action(parsed_args)

        self.assertEqual(returns, None)

    def test_cluster_template_delete_no_uuid(self):
        arglist = []
        verifylist = [('cluster-templates', [])]

        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)


class TestClusterTemplateList(TestClusterTemplate):
    attr = dict()
    attr['name'] = 'fake-ct-1'

    _cluster_template = (
        magnum_fakes.FakeClusterTemplate.create_one_cluster_template(attr))

    attr['name'] = 'fake-ct-2'

    _cluster_template2 = (
        magnum_fakes.FakeClusterTemplate.create_one_cluster_template(attr))

    columns = [
        'uuid',
        'name',
        'tags',
    ]

    datalist = (
        (_cluster_template.uuid, _cluster_template.name,
         _cluster_template.tags),
        (_cluster_template2.uuid, _cluster_template2.name,
         _cluster_template.tags)
    )

    def setUp(self):
        super(TestClusterTemplateList, self).setUp()

        self.cluster_templates_mock.list = mock.Mock()
        self.cluster_templates_mock.list.return_value = [
            self._cluster_template, self._cluster_template2
        ]

        # Get the command object to test
        self.cmd = osc_ct.ListTemplateCluster(self.app, None)

    def test_cluster_template_list_no_options(self):
        arglist = []
        verifylist = [
            ('limit', None),
            ('sort_key', None),
            ('sort_dir', None),
            ('fields', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.list.assert_called_with(
            limit=None,
            sort_dir=None,
            sort_key=None,
        )
        self.assertEqual(self.columns, columns)
        index = 0
        for d in data:
            self.assertEqual(self.datalist[index], d)
            index += 1

    def test_cluster_template_list_options(self):
        arglist = [
            '--limit', '1',
            '--sort-key', 'key',
            '--sort-dir', 'asc',
            '--fields', 'field1,field2'
        ]
        verifylist = [
            ('limit', 1),
            ('sort_key', 'key'),
            ('sort_dir', 'asc'),
            ('fields', 'field1,field2'),
        ]
        verifycolumns = self.columns + ['field1', 'field2']
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.list.assert_called_with(
            limit=1,
            sort_dir='asc',
            sort_key='key',
        )
        self.assertEqual(verifycolumns, columns)

    def test_cluster_template_list_bad_sort_dir_fail(self):
        arglist = [
            '--sort-dir', 'foo'
        ]
        verifylist = [
            ('limit', None),
            ('sort_key', None),
            ('sort_dir', 'foo'),
            ('fields', None),
        ]

        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)


class TestClusterTemplateShow(TestClusterTemplate):
    attr = dict()
    attr['name'] = 'fake-ct-1'

    _cluster_template = (
        magnum_fakes.FakeClusterTemplate.create_one_cluster_template(attr))

    columns = osc_ct.CLUSTER_TEMPLATE_ATTRIBUTES

    def setUp(self):
        super(TestClusterTemplateShow, self).setUp()

        datalist = map(lambda x: getattr(self._cluster_template, x),
                       self.columns)

        self.show_data_list = tuple(map(lambda x: x if x is not None else '-',
                                        datalist))

        self.cluster_templates_mock.get = mock.Mock()
        self.cluster_templates_mock.get.return_value = self._cluster_template

        # Get the command object to test
        self.cmd = osc_ct.ShowClusterTemplate(self.app, None)

    def test_cluster_template_show(self):
        arglist = ['fake-ct-1']
        verifylist = [('cluster-template', 'fake-ct-1')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.get.assert_called_with('fake-ct-1')
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.show_data_list, data)

    def test_cluster_template_show_no_ct_fail(self):
        arglist = []
        verifylist = []

        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)


class TestClusterTemplateUpdate(TestClusterTemplate):

    def setUp(self):
        super(TestClusterTemplateUpdate, self).setUp()

        attr = dict()
        attr['name'] = 'fake-ct-1'
        ct = magnum_fakes.FakeClusterTemplate.create_one_cluster_template(attr)

        self.cluster_templates_mock.update = mock.Mock()
        self.cluster_templates_mock.update.return_value = ct

        # Get the command object to test
        self.cmd = osc_ct.UpdateClusterTemplate(self.app, None)

    def test_cluster_template_update_pass(self):
        arglist = ['foo', 'remove', 'bar']
        verifylist = [
            ('cluster-template', 'foo'),
            ('op', 'remove'),
            ('attributes', [['bar']])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.update.assert_called_with(
            'foo',
            [{'op': 'remove', 'path': '/bar'}]
        )

    def test_cluster_template_update_hidden(self):
        arglist = ['foo', '--hidden']
        verifylist = [
            ('cluster-template', 'foo'),
            ('hidden', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.update.assert_called_with(
            'foo',
            [{'op': 'replace', 'path': '/hidden', 'value': True}]
        )

    def test_cluster_template_update_visible(self):
        arglist = ['foo', '--visible']
        verifylist = [
            ('cluster-template', 'foo'),
            ('hidden', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.update.assert_called_with(
            'foo',
            [{'op': 'replace', 'path': '/hidden', 'value': False}]
        )

    def test_cluster_template_update_with_op_and_hidden(self):
        arglist = ['foo', 'replace', 'name=bar', '--hidden']
        verifylist = [
            ('cluster-template', 'foo'),
            ('op', 'replace'),
            ('hidden', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.cluster_templates_mock.update.assert_called_with(
            'foo',
            [{'op': 'replace', 'path': '/name', 'value': 'bar'},
             {'op': 'replace', 'path': '/hidden', 'value': True}]
        )

    def test_cluster_template_update_no_args(self):
        arglist = ['foo']
        verifylist = [('cluster-template', 'foo')]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            InvalidAttribute,
            self.cmd.take_action, parsed_args)

    def test_cluster_template_update_bad_op(self):
        arglist = ['foo', 'bar', 'snafu']
        verifylist = [
            ('cluster-template', 'foo'),
            ('op', 'bar'),
            ('attributes', ['snafu'])
        ]

        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)
